var bv = BrainVoyager; // use short symbol to access BrainVoyager API

var scriptObj = new Object;bv

scriptObj.scriptNameForUser = "ASLVolumePlugin";    // NOT relevant for Plugin GUI script - specify name by which script will be shown to user (e.g. in menu)

scriptObj.dialogFileName   = "ASLVolumePlugin.ui";   // specify name of file that will be used by BV to create your UI
scriptObj.dialogAccessName = "PluginDialog";       // specify the name you want to use for accessing your UI

scriptObj.initDlg = function()
{
	var dlg = this.PluginDialog; // less to write

	dlg.windowTitle = "ASL Perfusion Volume Data Processing"; // use to change dialog title

	// connec UI elements to script funcctions - you need to know the names of your elements (check your definition)

	// Spatial transformation settings and auto-fill (to be implemented)
	var NativeSpaceRadioButton=bv.FindChild("NativeSpaceRadioButton");
	NativeSpaceRadioButton.clicked.connect(this, this.onNative);
	var ACPCSpaceRadioButton=bv.FindChild("ACPCSpaceRadioButton");
	ACPCSpaceRadioButton.clicked.connect(this, this.onACPC);
	var TalSpaceRadioButton=bv.FindChild("TalSpaceRadioButton");
	TalSpaceRadioButton.clicked.connect(this, this.onTAL);
	var AutoFillPushButton=bv.FindChild("AutoFillPushButton");
	AutoFillPushButton.clicked.connect(this, this.onAutoFill);

	var ASLFileLoadPushButton=bv.FindChild("ASLFileLoadPushButton");
	ASLFileLoadPushButton.clicked.connect(this, this.onLoadASL);
	
	var CALFileLoadPushButton=bv.FindChild("CALFileLoadPushButton");
	CALFileLoadPushButton.clicked.connect(this,this.onLoadCAL);

	var IATRFLoadPushButton=bv.FindChild("IATRFLoadPushButton");
	IATRFLoadPushButton.clicked.connect(this,this.onLoadIATRF);

	var FATRFLoadPushButton=bv.FindChild("FATRFLoadPushButton");
	FATRFLoadPushButton.clicked.connect(this,this.onLoadFATRF);

	var ACPCTRFLoadPushButton=bv.FindChild("ACPCTRFLoadPushButton");
	ACPCTRFLoadPushButton.clicked.connect(this,this.onLoadACPCTRF);

	var TALLoadPushButton=bv.FindChild("TALLoadPushButton");
	TALLoadPushButton.clicked.connect(this,this.onLoadTAL);

	var MSKLoadPushButton=bv.FindChild("MSKLoadPushButton");
	MSKLoadPushButton.clicked.connect(this,this.onLoadMSK);

	var GoPushButton=bv.FindChild("GoPushButton");
	GoPushButton.clicked.connect(this, this.OnGo);

	var QuitPushButton=bv.FindChild("QuitPushButton");
	QuitPushButton.clicked.connect(this, this.onQuit);

	var ClearPushButton=bv.FindChild("ClearPushButton");
	ClearPushButton.clicked.connect(this, this.onClear);

	var LoadSettingsLineEdit=bv.FindChild("LoadSettingsLineEdit");
	var LoadSettingsPushButton=bv.FindChild("LoadSettingsPushButton");
	LoadSettingsPushButton.clicked.connect(this, this.onLoadSettingsFile);

	var SaveSettingsPushButton=bv.FindChild("SaveSettingsPushButton");
	SaveSettingsPushButton.clicked.connect(this, this.onSaveSettingsFile);
	
	// Sets default FMR and VMR if already loaded and check last calculations for default settings ...
	bv.SetPluginStringParameter("ImportSettingsFile", LoadSettingsLineEdit.text);
	bv.SetPluginStringParameter("Command", "SetDefaultFiles");
	bv.ExecutePlugin();
	
	var VMRFileNameStr = bv.GetPluginStringParameter("VMRFileName");
	if (VMRFileNameStr=="") // No VMR is currently loaded ...
		dlg.close();

	var PASLRadioButton=bv.FindChild("PASLRadioButton");
	var CASLRadioButton=bv.FindChild("CASLRadioButton");

	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	var TI1Label=bv.FindChild("TI1Label");
	var TI2Label=bv.FindChild("TI2Label");
	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");

	var T1GMdoubleSpinBox=bv.FindChild("T1GMdoubleSpinBox");
	T1GMdoubleSpinBox.value=bv.GetPluginFloatParameter("T1GM");
	var T1ABdoubleSpinBox=bv.FindChild("T1ABdoubleSpinBox");
	T1ABdoubleSpinBox.value=bv.GetPluginFloatParameter("T1AB");
	var T2GMdoubleSpinBox=bv.FindChild("T2GMdoubleSpinBox");
	T2GMdoubleSpinBox.value=bv.GetPluginFloatParameter("T2GM");

	var TagEfficiencyDoubleSpinBox=bv.FindChild("TagEfficiencyDoubleSpinBox");
	TagEfficiencyDoubleSpinBox.value=bv.GetPluginFloatParameter("TagEff");
	var PartCoeffGMdoubleSpinBox=bv.FindChild("PartCoeffGMdoubleSpinBox");
	PartCoeffGMdoubleSpinBox.value=bv.GetPluginFloatParameter("PartCoeffGM");
	//var dMThrdoubleSpinBox=bv.FindChild("dMThrdoubleSpinBox");
	//dMThrdoubleSpinBox.value=bv.GetPluginFloatParameter("dMThr");
	var PartCoeffWMdoubleSpinBox=bv.FindChild("PartCoeffWMdoubleSpinBox");
	PartCoeffWMdoubleSpinBox.value=bv.GetPluginFloatParameter("PartCoeffWM");

	// Here read the default from "saved" settings ...
	PASLRadioButton.clicked.connect(this, this.onPASLClick);
	CASLRadioButton.clicked.connect(this, this.onCASLClick);

	PASLRadioButton.checked=bv.GetPluginIntParameter("PASL");
	CASLRadioButton.checked=bv.GetPluginIntParameter("CASL");
	if (PASLRadioButton.checked)
	{
		TRdoubleSpinBox.value=bv.GetPluginFloatParameter("TR");
		TEdoubleSpinBox.value=bv.GetPluginFloatParameter("TE");
		TI1Label.text="TI1:";
		TI2Label.text="TI2:";
		TI1doubleSpinBox.value=bv.GetPluginFloatParameter("TI1");
		TI2doubleSpinBox.value=bv.GetPluginFloatParameter("TI2");
	}
	if (CASLRadioButton.checked)
	{
		TRdoubleSpinBox.value=bv.GetPluginFloatParameter("TR");
		TEdoubleSpinBox.value=bv.GetPluginFloatParameter("TE");
		TI1Label.text="width:";
		TI2Label.text="delay:";
		TI1doubleSpinBox.value=bv.GetPluginFloatParameter("TI1");
		TI2doubleSpinBox.value=bv.GetPluginFloatParameter("TI2");
	}

	/// Setting for the input tab ...
	var ASLFileLineEdit = bv.FindChild("ASLFileLineEdit");
	ASLFileLineEdit.text = bv.GetPluginStringParameter("ASLFileName");

	var T2sFileLineEdit = bv.FindChild("T2sFileLineEdit");
	var DMFileLineEdit = bv.FindChild("DMFileLineEdit");
	var RELFileLineEdit = bv.FindChild("RELFileLineEdit");	
	var ABSFileLineEdit = bv.FindChild("ABSFileLineEdit");


	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	
	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");

	var SpatTranf3DGropBox = bv.FindChild("SpatTranf3DGropBox");

	if (VMRFileNameStr=="") // No VMR is currently loaded ...
	{
		SpatTranf3DGropBox.setEnabled(false);
		IATRFFileLineEdit.setEnabled(false);
		FATRFFileLineEdit.setEnabled(false);
		IATRFLoadPushButton.enabled=false;
		FATRFLoadPushButton.enabled=false;
		ACPCTRFFileLineEdit.setEnabled(false);
		ACPCTRFLoadPushButton.enabled=false;
		TALFileLineEdit.setEnabled(false);
		TALLoadPushButton.enabled=false;
		MSKFileLineEdit.setEnabled(false);
		MSKLoadPushButton.enabled=false;
	}
	else
	{
		SpatTranf3DGropBox.setEnabled(true);
		IATRFFileLineEdit.setEnabled(true);
		FATRFFileLineEdit.setEnabled(true);
		IATRFLoadPushButton.enabled=true;
		FATRFLoadPushButton.enabled=true;
		ACPCTRFFileLineEdit.setEnabled(false);
		ACPCTRFLoadPushButton.enabled=false;
		TALFileLineEdit.setEnabled(false);
		TALLoadPushButton.enabled=false;
		MSKFileLineEdit.setEnabled(true);
		MSKLoadPushButton.enabled=true;

		T2sFileLineEdit.text = bv.GetPluginStringParameter("T2sFileName");
		DMFileLineEdit.text = bv.GetPluginStringParameter("dMFileName");
		RELFileLineEdit.text = bv.GetPluginStringParameter("rCBFFileName");
		ABSFileLineEdit.text = bv.GetPluginStringParameter("aCBFFileName");
		
	}

	var ASLTabWidget = bv.FindChild("ASLTabWidget");
	ASLTabWidget.currentIndex=0;

}

scriptObj.onPASLClick = function()
{
	var dlg = this.PluginDialog;

	var PASLRadioButton=bv.FindChild("PASLRadioButton");
	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	var TI1Label=bv.FindChild("TI1Label");
	var TI2Label=bv.FindChild("TI2Label");
	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");
	
	if (PASLRadioButton.checked)
	{

		TRdoubleSpinBox.value=2500;
		TEdoubleSpinBox.value=13;
		TI1Label.text="TI1:";
		TI2Label.text="TI2:";
		TI1doubleSpinBox.value=700;
		TI2doubleSpinBox.value=1400;
	}
}

scriptObj.onCASLClick = function()
{
	var dlg = this.PluginDialog;
	
	var CASLRadioButton=bv.FindChild("CASLRadioButton");
	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	var TI1Label=bv.FindChild("TI1Label");
	var TI2Label=bv.FindChild("TI2Label");
	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");
	
	if (CASLRadioButton.checked)
	{
		TRdoubleSpinBox.value=3500;
		TEdoubleSpinBox.value=12;
		TI1Label.text="width:";
		TI2Label.text="delay:";
		TI1doubleSpinBox.value=1500;
		TI2doubleSpinBox.value=1000;
	}

}
		
scriptObj.onLoadASL = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadASL");
	bv.ExecutePlugin();

	var ASLFileLineEdit = bv.FindChild("ASLFileLineEdit");
	ASLFileLineEdit.text = bv.GetPluginStringParameter("ASLFileName"); // get result from plugin

	var T2sFileLineEdit = bv.FindChild("T2sFileLineEdit");
	T2sFileLineEdit.text = bv.GetPluginStringParameter("T2sFileName");

	var DMFileLineEdit = bv.FindChild("DMFileLineEdit");
	DMFileLineEdit.text = bv.GetPluginStringParameter("dMFileName");

	var RELFileLineEdit = bv.FindChild("RELFileLineEdit");
	RELFileLineEdit.text = bv.GetPluginStringParameter("rCBFFileName");

	var ABSFileLineEdit = bv.FindChild("ABSFileLineEdit");
	ABSFileLineEdit.text = bv.GetPluginStringParameter("aCBFFileName");
	
	var docFMR;
	docFMR = bv.OpenDocument(ASLFileLineEdit.text);
	if(docFMR == undefined)
	{
		BrainVoyager.PrintToLog("// ASL Plugin> FMR not specified, incorrect or not found.");
		return;
	}
	docFMR.Close(); // docFMR.Remove();  // close or remove input FMR
}

scriptObj.onLoadCAL = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadCAL");
	bv.ExecutePlugin();	

	var CALFileLineEdit=bv.FindChild("CALFileLineEdit");
	CALFileLineEdit.text = bv.GetPluginStringParameter("CALFileName");
	if(CALFileLineEdit.text=="")
	{
		CALFileLineEdit.setEnabled(false);
	}
	else
	{
		CALFileLineEdit.setEnabled(true);
	}

	var docFMR;
	docFMR = bv.OpenDocument(CALFileLineEdit.text);
	if(docFMR == undefined)
	{
		BrainVoyager.PrintToLog("// ASL Plugin> FMR not specified, incorrect or not found! Exiting plugin ...");
		return;
	}
	docFMR.Close(); // docFMR.Remove();  // close or remove input FMR
}

scriptObj.onLoadIATRF = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadIATRF");
	bv.ExecutePlugin();

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var VoxRes2RadioButton = bv.FindChild("VoxRes2RadioButton");
	var VoxRes3RadioButton = bv.FindChild("VoxRes3RadioButton");

	IATRFFileLineEdit.text = bv.GetPluginStringParameter("IATRFFileName"); // get result from plugin
	if(IATRFFileLineEdit.text=="")
	{
		//IATRFFileLineEdit.setEnabled(false);
		VoxRes2RadioButton.setEnabled(false);
		VoxRes3RadioButton.setEnabled(false);
	}
	else
	{
		//IATRFFileLineEdit.setEnabled(true);
		VoxRes2RadioButton.setEnabled(true);
		VoxRes3RadioButton.setEnabled(true);
	}
	if(FATRFFileLineEdit.text=="")
	{
		//FATRFFileLineEdit.setEnabled(false);
		VoxRes2RadioButton.setEnabled(false);
		VoxRes3RadioButton.setEnabled(false);
	}
	else
	{
		//FATRFFileLineEdit.setEnabled(true);
		VoxRes2RadioButton.setEnabled(true);
		VoxRes3RadioButton.setEnabled(true);
	}
}

scriptObj.onLoadFATRF = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadFATRF");
	bv.ExecutePlugin();

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var VoxRes2RadioButton = bv.FindChild("VoxRes2RadioButton");
	var VoxRes3RadioButton = bv.FindChild("VoxRes3RadioButton");

	FATRFFileLineEdit.text = bv.GetPluginStringParameter("FATRFFileName"); // get result from plugin
	if(IATRFFileLineEdit.text=="")
	{
		//IATRFFileLineEdit.setEnabled(false);
		VoxRes2RadioButton.setEnabled(false);
		VoxRes3RadioButton.setEnabled(false);
	}
	else
	{
		//IATRFFileLineEdit.setEnabled(true);
		VoxRes2RadioButton.setEnabled(true);
		VoxRes3RadioButton.setEnabled(true);
	}
	if(FATRFFileLineEdit.text=="")
	{
		//FATRFFileLineEdit.setEnabled(false);
		VoxRes2RadioButton.setEnabled(false);
		VoxRes3RadioButton.setEnabled(false);
	}
	else
	{
		//FATRFFileLineEdit.setEnabled(true);
		VoxRes2RadioButton.setEnabled(true);
		VoxRes3RadioButton.setEnabled(true);
	}
}

scriptObj.onLoadACPCTRF = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadACPCTRF");

	bv.ExecutePlugin();
	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	ACPCTRFFileLineEdit.text = bv.GetPluginStringParameter("ACPCTRFFileName"); // get result from plugin
}

scriptObj.onLoadTAL = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadTAL");

	bv.ExecutePlugin();
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	TALFileLineEdit.text = bv.GetPluginStringParameter("TALFileName"); // get result from plugin
}

scriptObj.onLoadMSK = function()
{
	var dlg = this.PluginDialog;

	bv.SetPluginStringParameter("Command", "LoadMSK");

	bv.ExecutePlugin();
	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	MSKFileLineEdit.text = bv.GetPluginStringParameter("MSKFileName"); // get result from plugin
}

scriptObj.OnGo = function()
{
	var dlg = this.PluginDialog;
	dlg.close();	

	bv.SetPluginStringParameter("Command", "CalculatePerfusion");

	// Input for rCBF
	var ASLFileLineEdit = bv.FindChild("ASLFileLineEdit");
	bv.SetPluginStringParameter("ASLFileName",  ASLFileLineEdit.text);

	var OddRadioButton = bv.FindChild("OddRadioButton");
	bv.SetPluginIntParameter("ControlOdd", OddRadioButton.checked);

	var LinInterpRadioButton = bv.FindChild("LinInterpRadioButton");
	if (LinInterpRadioButton.checked)
	{
		bv.SetPluginIntParameter("IntType", 1);
	}
	var CubicSplineIntRadioButton = bv.FindChild("CubicSplineIntRadioButton");
	if (CubicSplineIntRadioButton.checked)
	{
		bv.SetPluginIntParameter("IntType", 2);
	}
	var SincInterpRadioButton = bv.FindChild("SincInterpRadioButton");
	if (SincInterpRadioButton.checked)
	{
		bv.SetPluginIntParameter("IntType", 3);
	}
	
	var FMRBiasDoubleSpinBox = bv.FindChild("FMRBiasDoubleSpinBox");
	bv.SetPluginFloatParameter("FMRBias", FMRBiasDoubleSpinBox.value);

	var VMPPSCScalingBox = bv.FindChild("VMPPSCScalingBox");
	bv.SetPluginIntParameter("PSCScaling", VMPPSCScalingBox.checked);

	// Input for aCBF
	var CALFileLineEdit = bv.FindChild("CALFileLineEdit");
	bv.SetPluginStringParameter("CALFileName",  CALFileLineEdit.text);

	var PASLRadioButton = bv.FindChild("PASLRadioButton");
	bv.SetPluginIntParameter("PASL", PASLRadioButton.checked);
	
	var CASLRadioButton = bv.FindChild("CASLRadioButton");
	bv.SetPluginIntParameter("CASL", CASLRadioButton.checked);

	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	bv.SetPluginFloatParameter("TR", TRdoubleSpinBox.value);

	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	bv.SetPluginFloatParameter("TE", TEdoubleSpinBox.value);

	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	bv.SetPluginFloatParameter("TI1", TI1doubleSpinBox.value);

	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");
	bv.SetPluginFloatParameter("TI2", TI2doubleSpinBox.value);

	var PASLRadioButton=bv.FindChild("PASLRadioButton");
	bv.SetPluginIntParameter("PASL", PASLRadioButton.checked);

	var CASLRadioButton=bv.FindChild("CASLRadioButton");
	bv.SetPluginIntParameter("CASL", CASLRadioButton.checked);

	var TagEfficiencyDoubleSpinBox=bv.FindChild("TagEfficiencyDoubleSpinBox");
	bv.SetPluginFloatParameter("alpha", TagEfficiencyDoubleSpinBox.value);

	var PartCoeffGMdoubleSpinBox=bv.FindChild("PartCoeffGMdoubleSpinBox");
	bv.SetPluginFloatParameter("lambda_gm", PartCoeffGMdoubleSpinBox.value);

	var PartCoeffWMdoubleSpinBox=bv.FindChild("PartCoeffWMdoubleSpinBox");
	bv.SetPluginFloatParameter("lambda_wm", PartCoeffWMdoubleSpinBox.value);

	var T2GMdoubleSpinBox=bv.FindChild("T2GMdoubleSpinBox");
	bv.SetPluginFloatParameter("T2s_gm", T2GMdoubleSpinBox.value);
	
	var T1GMdoubleSpinBox=bv.FindChild("T1GMdoubleSpinBox");
	bv.SetPluginFloatParameter("T1_gm", T1GMdoubleSpinBox.value);
	
	var T1ABdoubleSpinBox=bv.FindChild("T1ABdoubleSpinBox");
	bv.SetPluginFloatParameter("T1A", T1ABdoubleSpinBox.value);
	
	//var dMThrdoubleSpinBox=bv.FindChild("dMThrdoubleSpinBox");
	//bv.SetPluginFloatParameter("dMThr", dMThrdoubleSpinBox.value);

	var docASLFMR;
	docASLFMR = bv.OpenDocument(ASLFileLineEdit.text);
	if(docASLFMR == undefined)
	{
		ASLFileLineEdit.text = "";
		BrainVoyager.PrintToLog("// ASL Plugin> ASL FMR not specified, incorrect or not found.");
		return;
	}

	var docCALFMR;
	if(CALFileLineEdit.text=="")
	{
		BrainVoyager.PrintToLog("// ASL Plugin> CAL FMR not provided.");
	}
	else
	{
		docCALFMR = bv.OpenDocument(CALFileLineEdit.text);
		if(docCALFMR == undefined)
		{
			CALFileLineEdit.text = "";
			BrainVoyager.PrintToLog("// ASL Plugin> CAL FMR not incorrect or not found.");
			return;
		}
	}

	bv.ExecutePlugin();

	docASLFMR.Close();  // close or remove input FMR
	if(CALFileLineEdit.text!="")
	{
		docCALFMR.Close();  // close or remove input FMR
	}

	var SpatTranf3DGropBox = bv.FindChild("SpatTranf3DGropBox");
	var NativeSpaceRadioButton=bv.FindChild("NativeSpaceRadioButton");
	var ACPCSpaceRadioButton=bv.FindChild("ACPCSpaceRadioButton");
	var TalSpaceRadioButton=bv.FindChild("TalSpaceRadioButton");
	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var IATRFLoadPushButton = bv.FindChild("IATRFLoadPushButton");
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var FATRFLoadPushButton = bv.FindChild("FATRFLoadPushButton");
	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	var ACPCTRFLoadPushButton = bv.FindChild("ACPCTRFLoadPushButton");
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	var TALLoadPushButton = bv.FindChild("TALLoadPushButton");
	
	if (SpatTranf3DGropBox.enabled==false)
	{
		// No VTCs are to be calculated so, show new FMRs
		// docFMR = bv.OpenDocument("rCBF.fmr");
		if (dlg.InputDataGroupBox.CALFileLineEdit.text!="")
		{
			//docFMR = bv.OpenDocument("aCBF.fmr");
		}
		return;
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Create VTCs
	//BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTCs with T2s and Perfusion-weighted images and maps...");
	//..., nameVTCinTAL, dataType, resolution, interpolation, threshold);
	var dataType = 2; // 1: int16, 2: float32
	var resolution = 2; // one of 1, 2 or 3 mm
	var interpolation = 1; // ?
	var threshold = 0; 
	var extendedBoundingBox = false;

	if (dlg.RunGroupBox.VoxRes2RadioButton.checked)
		resolution=2;
	if (dlg.RunGroupBox.VoxRes3RadioButton.checked)
		resolution=3;
	
	var docVMR = bv.ActiveDocument;
	docVMR.ExtendedTALSpaceForVTCCreation = extendedBoundingBox;

	//BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the T2s-weighted (BOLD) image time-series ...");
	var success_T2s;
	if (TALFileLineEdit.text=="")
	{
		if (ACPCTRFFileLineEdit.text=="")
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the T2s-weighted (BOLD) image time-series in native space...");
			success_T2s = docVMR.CreateVTCInVMRSpace("T2s.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
														"T2s.vtc", dataType, resolution, interpolation, threshold);
		}
		else
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the T2s-weighted (BOLD) image time-series in ACPC space...");
			success_T2s = docVMR.CreateVTCInACPCSpace("T2s.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
															ACPCTRFFileLineEdit.text,
														"T2s.vtc", dataType, resolution, interpolation, threshold);
		}
	}
	else
	{
		BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the T2s-weighted (BOLD) image time-series in Talairach space...");
		success_T2s = docVMR.CreateVTCInTALSpace("T2s.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
																ACPCTRFFileLineEdit.text, TALFileLineEdit.text,
													"T2s.vtc", dataType, resolution, interpolation, threshold);
	}
	if (!success_T2s)
		return;
	
	//BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the Perfusion-weighted (dM) image time-series ...");
	var success_DM;
	if (TALFileLineEdit.text=="")
	{
		if (ACPCTRFFileLineEdit.text=="")
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the Perfusion-weighted (dM) image time-series in native space ...");
			success_DM = docVMR.CreateVTCInVMRSpace("dM.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
														"dM.vtc", dataType, resolution, interpolation, threshold);
		}
		else
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the Perfusion-weighted (dM) image time-series in ACPC space ...");
			success_DM = docVMR.CreateVTCInACPCSpace("dM.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
															ACPCTRFFileLineEdit.text,
														"dM.vtc", dataType, resolution, interpolation, threshold);
		}
	}
	else
	{
		BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC for the Perfusion-weighted (dM) image time-series in Talairach space...");
		success_DM = docVMR.CreateVTCInTALSpace("dM.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
																ACPCTRFFileLineEdit.text, TALFileLineEdit.text,
													"dM.vtc", dataType, resolution, interpolation, threshold);
	}
	if (!success_DM)
		return;

	//BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the rCBF map as a first volume ...");
	var success_rCBF;
	if (TALFileLineEdit.text=="")
	{
		if (ACPCTRFFileLineEdit.text=="")
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the rCBF map as a first volume in native space ...");
			success_rCBF = docVMR.CreateVTCInVMRSpace("rCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
														"rCBF.vtc", dataType, resolution, interpolation, threshold);
		}
		else
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the rCBF map as a first volume in ACPC space ...");
			success_rCBF = docVMR.CreateVTCInACPCSpace("rCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
															ACPCTRFFileLineEdit.text, "rCBF.vtc", dataType, resolution, interpolation, threshold);
		}
	}
	else
	{	
		BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the rCBF map as a first volume in Talairach space ...");
		success_rCBF = docVMR.CreateVTCInTALSpace("rCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
																ACPCTRFFileLineEdit.text, TALFileLineEdit.text,
													"rCBF.vtc", dataType, resolution, interpolation, threshold);
	}
	if (!success_rCBF)
		return;

	var perform_aCBF;
	if(CALFileLineEdit.text=="")
	{
		perform_aCBF = 0;
	}
	else
	{
		perform_aCBF = 1;
	}

	if (perform_aCBF)
	{
		BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the aCBF map as a first volume ...");
	}

	var success_aCBF=0;
	if (perform_aCBF)
	{
		if (TALFileLineEdit.text=="")
		{
			if (ACPCTRFFileLineEdit.text=="")
			{
				BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the aCBF map as a first volume in native space...");
				success_aCBF = docVMR.CreateVTCInVMRSpace("aCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
														"aCBF.vtc", dataType, resolution, interpolation, threshold);
			}
			else
			{
				BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the aCBF map as a first volume in ACPC space...");
				success_aCBF = docVMR.CreateVTCInACPCSpace("aCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
																ACPCTRFFileLineEdit.text, "aCBF.vtc", dataType, resolution, interpolation, threshold);
			}
		}
		else
		{
			BrainVoyager.PrintToLog("// ASL Plugin> Creating the VTC with the aCBF map as a first volume in Talairach space ...");
			success_aCBF = docVMR.CreateVTCInTALSpace("aCBF.fmr", IATRFFileLineEdit.text, FATRFFileLineEdit.text,
																	ACPCTRFFileLineEdit.text, TALFileLineEdit.text,
														"aCBF.vtc", dataType, resolution, interpolation, threshold);
		}
	}

	var docFMR;
	docFMR = BrainVoyager.OpenDocument("T2s.fmr");
	docFMR.Remove();
	docFMR = BrainVoyager.OpenDocument("dM.fmr");
	docFMR.Remove();
	docFMR = BrainVoyager.OpenDocument("rCBF.fmr");
	docFMR.Remove();

	if (perform_aCBF)
	{
		docFMR = BrainVoyager.OpenDocument("aCBF.fmr");
 		docFMR.Remove();
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	var T2sFileLineEdit = bv.FindChild("T2sFileLineEdit");
	bv.SetPluginStringParameter("T2sVTCFileName", T2sFileLineEdit.text);

	var DMFileLineEdit = bv.FindChild("DMFileLineEdit");
	bv.SetPluginStringParameter("dMVTCFileName", DMFileLineEdit.text); 

	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	bv.SetPluginStringParameter("MaskFileName", MSKFileLineEdit.text);

	var RELFileLineEdit = bv.FindChild("RELFileLineEdit");
	bv.SetPluginStringParameter("rCBFVMPFileName", RELFileLineEdit.text);

	var ABSFileLineEdit = bv.FindChild("ABSFileLineEdit");
	bv.SetPluginStringParameter("aCBFVMPFileName", ABSFileLineEdit.text);
	

	bv.SetPluginStringParameter("Command", "FinalizeVTCs");
	if (success_T2s && success_DM)
	{
		bv.ExecutePlugin();
	}
	else
	{
		T2sFileLineEdit.text="";
		DMFileLineEdit.text="";
	}
		
	if (perform_aCBF)
	{
		bv.SetPluginStringParameter("aCBFVMPFileName", ABSFileLineEdit.text); 
	}
	else
	{
		bv.SetPluginStringParameter("aCBFVMPFileName", "");
	}

	bv.SetPluginStringParameter("Command", "FinalizeVMPs");
	if (success_rCBF)
	{
		bv.ExecutePlugin();
	}
	else
	{
		RELFileLineEdit.text="";
		ABSFileLineEdit.text="";
	}

	/////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////
	
	/////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////

	BrainVoyager.PrintToLog("// ASL PLugin>  ASL Plugin completed.");

	/////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////

}

scriptObj.onNative = function()
{
	var dlg = this.PluginDialog;

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var IATRFLoadPushButton = bv.FindChild("IATRFLoadPushButton");
	
	IATRFFileLineEdit.setEnabled(true);
	IATRFLoadPushButton.enabled=true;
	
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var FATRFLoadPushButton = bv.FindChild("FATRFLoadPushButton");

	FATRFFileLineEdit.setEnabled(true);
	FATRFLoadPushButton.enabled=true;

	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	var ACPCTRFLoadPushButton = bv.FindChild("ACPCTRFLoadPushButton");

	ACPCTRFFileLineEdit.setEnabled(false);
	ACPCTRFLoadPushButton.enabled=false;
	
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	var TALLoadPushButton = bv.FindChild("TALLoadPushButton");
	
	TALFileLineEdit.setEnabled(false);
	TALLoadPushButton.enabled=false;
	
	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	var MSKLoadPushButton = bv.FindChild("MSKLoadPushButton");
	
	MSKFileLineEdit.setEnabled(true);
	MSKLoadPushButton.enabled=true;

}

scriptObj.onACPC = function()
{

	var dlg = this.PluginDialog;

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var IATRFLoadPushButton = bv.FindChild("IATRFLoadPushButton");
	
	IATRFFileLineEdit.setEnabled(true);
	IATRFLoadPushButton.enabled=true;
	
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var FATRFLoadPushButton = bv.FindChild("FATRFLoadPushButton");

	FATRFFileLineEdit.setEnabled(true);
	FATRFLoadPushButton.enabled=true;

	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	var ACPCTRFLoadPushButton = bv.FindChild("ACPCTRFLoadPushButton");

	ACPCTRFFileLineEdit.setEnabled(true);
	ACPCTRFLoadPushButton.enabled=true;
	
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	var TALLoadPushButton = bv.FindChild("TALLoadPushButton");
	
	TALFileLineEdit.setEnabled(false);
	TALLoadPushButton.enabled=false;
	
	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	var MSKLoadPushButton = bv.FindChild("MSKLoadPushButton");
	
	MSKFileLineEdit.setEnabled(true);
	MSKLoadPushButton.enabled=true;

}

scriptObj.onTAL = function()
{

	var dlg = this.PluginDialog;

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	var IATRFLoadPushButton = bv.FindChild("IATRFLoadPushButton");
	
	IATRFFileLineEdit.setEnabled(true);
	IATRFLoadPushButton.enabled=true;
	
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	var FATRFLoadPushButton = bv.FindChild("FATRFLoadPushButton");

	FATRFFileLineEdit.setEnabled(true);
	FATRFLoadPushButton.enabled=true;

	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	var ACPCTRFLoadPushButton = bv.FindChild("ACPCTRFLoadPushButton");

	ACPCTRFFileLineEdit.setEnabled(true);
	ACPCTRFLoadPushButton.enabled=true;
	
	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	var TALLoadPushButton = bv.FindChild("TALLoadPushButton");
	
	TALFileLineEdit.setEnabled(true);
	TALLoadPushButton.enabled=true;
	
	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	var MSKLoadPushButton = bv.FindChild("MSKLoadPushButton");
	
	MSKFileLineEdit.setEnabled(true);
	MSKLoadPushButton.enabled=true;

}

scriptObj.onAutoFill = function()
{
	// to be implemeneted??
}

scriptObj.onLoadSettingsFile = function()
{
	var dlg = this.PluginDialog;
	bv.SetPluginStringParameter("Command", "ImportSettingsFromFile");
	bv.ExecutePlugin();

	var LoadSettingsLineEdit=bv.FindChild("LoadSettingsLineEdit");
	LoadSettingsLineEdit.text = bv.GetPluginStringParameter("SettingsFileName");

	var PASLRadioButton=bv.FindChild("PASLRadioButton");
	var CASLRadioButton=bv.FindChild("CASLRadioButton");

	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	var TI1Label=bv.FindChild("TI1Label");
	var TI2Label=bv.FindChild("TI2Label");
	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");

	var T1GMdoubleSpinBox=bv.FindChild("T1GMdoubleSpinBox");
	T1GMdoubleSpinBox.value=bv.GetPluginFloatParameter("T1GM");
	var T1ABdoubleSpinBox=bv.FindChild("T1ABdoubleSpinBox");
	T1ABdoubleSpinBox.value=bv.GetPluginFloatParameter("T1AB");
	var T2GMdoubleSpinBox=bv.FindChild("T2GMdoubleSpinBox");
	T2GMdoubleSpinBox.value=bv.GetPluginFloatParameter("T2GM");

	var TagEfficiencyDoubleSpinBox=bv.FindChild("TagEfficiencyDoubleSpinBox");
	TagEfficiencyDoubleSpinBox.value=bv.GetPluginFloatParameter("TagEff");
	var PartCoeffGMdoubleSpinBox=bv.FindChild("PartCoeffGMdoubleSpinBox");
	PartCoeffGMdoubleSpinBox.value=bv.GetPluginFloatParameter("PartCoeffGM");
	var PartCoeffWMdoubleSpinBox=bv.FindChild("PartCoeffWMdoubleSpinBox");
	PartCoeffWMdoubleSpinBox.value=bv.GetPluginFloatParameter("PartCoeffWM");
	//var dMThrdoubleSpinBox=bv.FindChild("dMThrdoubleSpinBox");
	//dMThrdoubleSpinBox.value=bv.GetPluginFloatParameter("dMThr");

	// Here read the default from "saved" settings ...
	//PASLRadioButton.clicked.connect(this, this.onPASLClick);
	//CASLRadioButton.clicked.connect(this, this.onCASLClick);

	PASLRadioButton.checked=bv.GetPluginIntParameter("PASL");
	CASLRadioButton.checked=bv.GetPluginIntParameter("CASL");
	if (PASLRadioButton.checked)
	{
		TRdoubleSpinBox.value=bv.GetPluginFloatParameter("TR");
		TEdoubleSpinBox.value=bv.GetPluginFloatParameter("TE");
		TI1Label.text="TI1:";
		TI2Label.text="TI2:";
		TI1doubleSpinBox.value=bv.GetPluginFloatParameter("TI1");
		TI2doubleSpinBox.value=bv.GetPluginFloatParameter("TI2");
	}
	if (CASLRadioButton.checked)
	{
		TRdoubleSpinBox.value=bv.GetPluginFloatParameter("TR");
		TEdoubleSpinBox.value=bv.GetPluginFloatParameter("TE");
		TI1Label.text="width:";
		TI2Label.text="delay:";
		TI1doubleSpinBox.value=bv.GetPluginFloatParameter("TI1");
		TI2doubleSpinBox.value=bv.GetPluginFloatParameter("TI2");
	}

}
scriptObj.onSaveSettingsFile = function()
{

	var dlg = this.PluginDialog;
	

	bv.SetPluginStringParameter("Command", "ExportSettingsToFile");

	var ASLFileLineEdit=bv.FindChild("ASLFileLineEdit");
	bv.SetPluginStringParameter("ASLFileName", ASLFileLineEdit.text);

	var TRdoubleSpinBox=bv.FindChild("TRdoubleSpinBox");
	bv.SetPluginFloatParameter("TR", TRdoubleSpinBox.value);

	var TEdoubleSpinBox=bv.FindChild("TEdoubleSpinBox");
	bv.SetPluginFloatParameter("TE", TEdoubleSpinBox.value);

	var TI1doubleSpinBox=bv.FindChild("TI1doubleSpinBox");
	bv.SetPluginFloatParameter("TI1", TI1doubleSpinBox.value);

	var TI2doubleSpinBox=bv.FindChild("TI2doubleSpinBox");
	bv.SetPluginFloatParameter("TI2", TI2doubleSpinBox.value);

	var PASLRadioButton=bv.FindChild("PASLRadioButton");
	bv.SetPluginIntParameter("PASL", PASLRadioButton.checked);

	var CASLRadioButton=bv.FindChild("CASLRadioButton");
	bv.SetPluginIntParameter("CASL", CASLRadioButton.checked);

	var TagEfficiencyDoubleSpinBox=bv.FindChild("TagEfficiencyDoubleSpinBox");
	bv.SetPluginFloatParameter("alpha", TagEfficiencyDoubleSpinBox.value);

	var PartCoeffGMdoubleSpinBox=bv.FindChild("PartCoeffGMdoubleSpinBox");
	bv.SetPluginFloatParameter("lambda_gm", PartCoeffGMdoubleSpinBox.value);

	var PartCoeffWMdoubleSpinBox=bv.FindChild("PartCoeffWMdoubleSpinBox");
	bv.SetPluginFloatParameter("lambda_wm", PartCoeffWMdoubleSpinBox.value);

	var T2GMdoubleSpinBox=bv.FindChild("T2GMdoubleSpinBox");
	bv.SetPluginFloatParameter("T2s_gm", T2GMdoubleSpinBox.value);
	
	var T1GMdoubleSpinBox=bv.FindChild("T1GMdoubleSpinBox");
	bv.SetPluginFloatParameter("T1_gm", T1GMdoubleSpinBox.value);
	
	var T1ABdoubleSpinBox=bv.FindChild("T1ABdoubleSpinBox");
	bv.SetPluginFloatParameter("T1A", T1ABdoubleSpinBox.value);
	
	//var dMThrdoubleSpinBox=bv.FindChild("dMThrdoubleSpinBox");
	//bv.SetPluginFloatParameter("dMThr", dMThrdoubleSpinBox.value);

	bv.ExecutePlugin();
	
	var LoadSettingsLineEdit=bv.FindChild("LoadSettingsLineEdit");
	LoadSettingsLineEdit.text = bv.GetPluginStringParameter("SettingsFileName");

}

scriptObj.onClear = function()
{

	var dlg = this.PluginDialog;

	/// Setting for the input tab ...
	var ASLFileLineEdit = bv.FindChild("ASLFileLineEdit");
	ASLFileLineEdit.text = ""; //bv.GetPluginStringParameter("ASLFileName");

	var CALFileLineEdit = bv.FindChild("CALFileLineEdit");
	CALFileLineEdit.text="";

	var IATRFFileLineEdit = bv.FindChild("IATRFFileLineEdit");
	IATRFFileLineEdit.text="";
	
	var FATRFFileLineEdit = bv.FindChild("FATRFFileLineEdit");
	FATRFFileLineEdit.text="";
	
	var ACPCTRFFileLineEdit = bv.FindChild("ACPCTRFFileLineEdit");
	ACPCTRFFileLineEdit.text="";

	var TALFileLineEdit = bv.FindChild("TALFileLineEdit");
	TALFileLineEdit.text="";

	var MSKFileLineEdit = bv.FindChild("MSKFileLineEdit");
	MSKFileLineEdit.text="";

	var T2sFileLineEdit = bv.FindChild("T2sFileLineEdit");
	T2sFileLineEdit.text = "";
	var DMFileLineEdit = bv.FindChild("DMFileLineEdit");
	DMFileLineEdit.text = "";
	var RELFileLineEdit = bv.FindChild("RELFileLineEdit");
	RELFileLineEdit.text = "";
	var ABSFileLineEdit = bv.FindChild("ABSFileLineEdit");
	ABSFileLineEdit.text = "";

}

scriptObj.onQuit = function()
{
	this.PluginDialog.close();
}

returnScriptObj = function()
{
	return scriptObj; // it is important to return script object containing properties and functions to be called from BrainVoyager
}
returnScriptObj();
