var bv = BrainVoyager; // use short symbol to access BrainVoyager QX API
var scriptObj = new Object;

scriptObj.dialogFileName   = "MDSSPlotPlugin.ui";  // specify name of file that will be used by BV to create your UI
scriptObj.dialogAccessName = "PluginDialog";         // specify the name you want to use for accessing your UI

scriptObj.initDlg = function()
{
	var dlg = this.PluginDialog; // less to write

	dlg.windowTitle = "Multi-Dimensional Scaling Similarity Plot"; // use to change dialog title

	// connec UI elements to script funcctions - you need to know the names of your elements (check your definition)
	dlg.quitButton.clicked.connect(this, this.onQuit);
	dlg.plotButton.clicked.connect(this, this.onPlot);
	
	// First tab (input data)
	bv.FindChild("selectVMPFileBtn").clicked.connect(this, this.onSelectVMPFile);
	bv.FindChild("selectSMPFileBtn").clicked.connect(this, this.onSelectSMPFile);
	bv.FindChild("selectMSKFileBtn").clicked.connect(this, this.onSelectMSKFile);
	bv.FindChild("selectVOIFileBtn").clicked.connect(this, this.onSelectVOIFile);
	bv.FindChild("selectPOIFileBtn").clicked.connect(this, this.onSelectPOIFile);

	//bv.FindChild("MSKGroupBox").clicked.connect(this, this.onClickMSKBox);
	//bv.FindChild("ROIGroupBox").clicked.connect(this, this.onClickROIBox);

	// Second tab (class selection)
	bv.FindChild("ClassSelBtn").clicked.connect(this, this.onSelectAllClasses);
	bv.FindChild("ClassDesBtn").clicked.connect(this, this.onDeselectAllClasses);
	
	// Third tab (projection options)
	bv.FindChild("MMDSProjGrpBx").clicked.connect(this, this.onClickedMMDSProj);
	bv.FindChild("CCAProjGrpBx").clicked.connect(this, this.onClickedCCAProj);
	bv.FindChild("SAMProjGrpBx").clicked.connect(this, this.onClickedSAMProj);
	
	bv.SetPluginStringParameter("Command", "StartCheck");
	if( !bv.ExecutePlugin() ) // prereqs not met
		this.onQuit();

	var VMPFileName = bv.GetPluginStringParameter("VMPFileName");
	bv.PrintToLog("Script>  Selected VMP File: " + VMPFileName);
	var VMPEd  = bv.FindChild("VMPFileEd");
	VMPEd.text = VMPFileName;

	var SMPFileName = bv.GetPluginStringParameter("SMPFileName");
	bv.PrintToLog("Script>  Selected SMP File: " + SMPFileName);
	var SMPEd  = bv.FindChild("SMPFileEd");
	SMPEd.text = SMPFileName;

	var VOIFileName = bv.GetPluginStringParameter("VOIFileName");
	bv.PrintToLog("Script>  Selected VOI File: " + VOIFileName);
	var VOIEd  = bv.FindChild("VOIFileEd");
	VOIEd.text = VOIFileName;

	var POIFileName = bv.GetPluginStringParameter("POIFileName");
	bv.PrintToLog("Script>  Selected POI File: " + POIFileName);
	var POIEd  = bv.FindChild("POIFileEd");
	POIEd.text = POIFileName;

	// Adapting GUI to current mode ...
	var MODE = bv.GetPluginIntParameter("MODE"); // 1=VOL, 2=SUR

	if (MODE==1)
	{
		bv.PrintToLog("Script> Selected mode: Volume");

		bv.FindChild("selectVMPFileBtn").enabled=true;
		bv.FindChild("VMPFileEd").enabled=true;

		bv.FindChild("selectSMPFileBtn").enabled=false;
		bv.FindChild("SMPFileEd").enabled=false;

		bv.FindChild("selectMSKFileBtn").enabled=true;
		bv.FindChild("MSKFileEd").enabled=true;

		bv.FindChild("ROIGroupBox").enabled=true;

		bv.FindChild("VOIFileLb").enabled=true;
		bv.FindChild("VOIFileEd").enabled=true;
		bv.FindChild("selectVOIFileBtn").enabled=true;
		
		bv.FindChild("POIFileLb").enabled=false;
		bv.FindChild("POIFileEd").enabled=false;
		bv.FindChild("selectPOIFileBtn").enabled=false;

		this.fillVOIList();
		
	}

	if (MODE==2)
	{
		bv.PrintToLog("Script> Selected mode: Surface");

		bv.FindChild("selectVMPFileBtn").enabled=false;
		bv.FindChild("VMPFileEd").enabled=false;

		bv.FindChild("selectSMPFileBtn").enabled=true;
		bv.FindChild("SMPFileEd").enabled=true;

		bv.FindChild("selectMSKFileBtn").enabled=false;
		bv.FindChild("MSKFileEd").enabled=false;

		bv.FindChild("ROIGroupBox").enabled=true;

		bv.FindChild("VOIFileLb").enabled=false;
		bv.FindChild("VOIFileEd").enabled=false;
		bv.FindChild("selectVOIFileBtn").enabled=false;
		
		bv.FindChild("POIFileLb").enabled=true;
		bv.FindChild("POIFileEd").enabled=true;
		bv.FindChild("selectPOIFileBtn").enabled=true;

		this.fillPOIList();

	}

	bv.PrintToLog("// Script> Filling the class table...");
	this.fillClassTable();
	bv.PrintToLog("// Script> Filling the study table...");
	this.fillStudyTable();

	bv.FindChild("MSKGroupBox").checked=false;

	var ROILoaded = bv.GetPluginIntParameter("ROILoaded"); // 1=VOL, 2=SUR 
	if (ROILoaded==1)
		bv.FindChild("ROIGroupBox").checked=true;
	else
		bv.FindChild("ROIGroupBox").checked=false;

	// Choosing the current tab:
	var MDSSTabWidget = bv.FindChild("MDSSTabWidget");
	var MapFound = bv.GetPluginIntParameter("MapFound");
	if ( MapFound==0 )
		MDSSTabWidget.currentIndex=0;
	else
		MDSSTabWidget.currentIndex=1;

}

scriptObj.onQuit = function()
{
	this.PluginDialog.close();
}

scriptObj.onPlot = function()
{
	var dlg = this.PluginDialog; // less to write
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// This part is for ROI-based access ...
	var ROIsLW = bv.FindChild("ROIsLsWdg");	
	var n_rois = ROIsLW.count;
	if( (bv.FindChild("ROIGroupBox").checked) && (n_rois > 0) )
	{
		bv.SetPluginIntParameter("UseROI", 1);
		var sel_voi = ROIsLW.currentRow;
		if (sel_voi < 0)
		{
			bv.MessageBox("No ROI select - please select the ROI defining the multi-dimensional space for MDSS calculations!");
			return;
		}
		bv.SetPluginIntParameter("SelectedROIIndex", sel_voi); // tell plugin which ROI has been selected		
	}
	else
	{
		bv.SetPluginIntParameter("UseROI", 0);
		bv.SetPluginIntParameter("SelectedROIIndex", -1);
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Mask file
	var MSKEd  = bv.FindChild("MSKFileEd");
	var MSKFileName = MSKEd.text; 
	if (bv.FindChild("MSKGroupBox").checked)
	{
		bv.SetPluginIntParameter("UseMask", 1);
		if (MSKFileName=="")
		{
			// Mask not specified
			bv.SetPluginIntParameter("UseMask", 0);
		}
		else
		{
			bv.SetPluginIntParameter("UseMask", 1);
			bv.SetPluginStringParameter("MaskFileName", MSKFileName);
			bv.PrintToLog("Script> Selected MSK File: " + MSKFileName);
		}
	}
	else
	{
		bv.SetPluginIntParameter("UseMask", 0);
	}

	// Map preprocessing
	if (bv.FindChild("RMMCheckBox").checked)
	{
		bv.SetPluginIntParameter("RemoveMeanMap", 1);
		
	}
	else
	{
		bv.SetPluginIntParameter("RemoveMeanMap", 0);
	}

	if (bv.FindChild("RemPCChkBx").checked)
	{
		bv.SetPluginIntParameter("RemoveMapPCs", 1);
	}
	else
	{
		bv.SetPluginIntParameter("RemoveMapPCs", 0);
	}
	var nPCs = bv.FindChild("RemPCSpBx").value;
	bv.SetPluginIntParameter("nPCs", nPCs);

	// Settings from Tab 2 (class selection)
	var ClassTableWidget = bv.FindChild("ClassTableWidget");
	var NrOfClasses = ClassTableWidget.rowCount;
	bv.SetPluginIntParameter("NrOfClasses", NrOfClasses); // tell plugin if this CLASS has been selected...

	for (var i=0; i<NrOfClasses; i++)
	{
		var ClassSelStr = "SelectionClass-" + (i+1);
		if (ClassTableWidget.item(i,1).checkState()==Qt.Checked)
			bv.SetPluginIntParameter(ClassSelStr, 1); // tell plugin if this CLASS has been selected...
		else
			bv.SetPluginIntParameter(ClassSelStr, 0); // tell plugin if this CLASS has been selected...
	}

	// Settings from Tab 3 (group selection)
	var UseGPA = bv.FindChild("UseGPAChkBx").checked;
	bv.SetPluginIntParameter("UseGroupAssignment", UseGPA);

	var StudyTableWidget = bv.FindChild("StudyTableWidget");
	var NrOfStudies = StudyTableWidget.rowCount;
	bv.SetPluginIntParameter("NrOfStudies", NrOfStudies); // tell plugin if this CLASS has been selected...

	for (var i=0; i<NrOfStudies; i++)
	{
		bv.SetPluginStringParameter( "GPA-" + (i+1), StudyTableWidget.item(i,1).text() );
	}
	
	// Settings from Tab 4
	if (bv.FindChild("Sim2Dis1mDRadBtn").checked)
	{
		bv.SetPluginIntParameter("Sim2Dis", 0);
	}
	if (bv.FindChild("Sim2Dis1mSQRDRadBtn").checked)
	{
		bv.SetPluginIntParameter("Sim2Dis", 1);
		var p = bv.FindChild("Sim2DisPSpBx").value;
		bv.SetPluginIntParameter("p", p);
	}

	if (bv.FindChild("ckBxFDR").checked)
	{
		bv.SetPluginIntParameter("UseFDR", 1); // Use FDR
	}
	else
	{
		bv.SetPluginIntParameter("UseFDR", 0); // Use FDR
	}

	var qFDR = bv.FindChild("dbSpBxFDR").value;
	bv.SetPluginFloatParameter("qFDR", qFDR);

	if(bv.FindChild("MMDSProjGrpBx").checked)
	{
		bv.SetPluginIntParameter("ProjectionMethod", 1); // MMDS projection
		bv.PrintToLog("Script> Projection method: MMDS");
	}

	var dim1st=bv.FindChild("Dim1spBx").value;
	var dim2nd=bv.FindChild("Dim2spBx").value;
	bv.SetPluginIntParameter("dim1st", dim1st);
	bv.SetPluginIntParameter("dim2nd", dim2nd);
	
	var Alpha=0.0;
	var Radius=0.0;
	var Epochs=0;
	
	if(bv.FindChild("CCAProjGrpBx").checked)
	{
		bv.SetPluginIntParameter("ProjectionMethod", 2); // CCA projection
		bv.PrintToLog("Script> Projection method: CCA");
		Alpha = bv.FindChild("CCAAlphaDSpBx").value;
		Radius = bv.FindChild("CCARadiusDSpBx").value;
		Epochs = bv.FindChild("CCAEpochsSpBx").value;
	}

	if(bv.FindChild("SAMProjGrpBx").checked)
	{
		bv.SetPluginIntParameter("ProjectionMethod", 3); // SAM projection
		bv.PrintToLog("Script> Projection method: SAM");
		Alpha = bv.FindChild("SAMAlphaDSpBx").value;
		Epochs = bv.FindChild("SAMEpochsSpBx").value;
	}

	bv.SetPluginFloatParameter("Alpha", Alpha);
	bv.SetPluginFloatParameter("Radius", Radius);
	bv.SetPluginIntParameter("Epochs", Epochs);
	
	bv.SetPluginStringParameter("Command", "Plot");
	//this.PluginDialog.close();
	bv.ExecutePlugin();

	//var docVMR = bv.ActiveDocument;	
	//docVMR.ShowVolumeMapsDlg();

}

scriptObj.onSelectVOIFile = function()
{
	var VOIFileName = bv.BrowseFile("Select VOI File", "POI files (*.voi)");
	bv.PrintToLog("Script>  Selected VOI File: " + VOIFileName);	
	bv.SetPluginStringParameter("VOIFileName", VOIFileName);
	bv.SetPluginStringParameter("Command", "SelectVOIFile");

	if(VOIFileName == "")
	{
		bv.FindChild("ROIGroupBox").checked=false;
		bv.SetPluginStringParameter("VOIFileName", "");
		bv.FindChild("ROIsLsWdg").clear();
	}
	if (VOIFileName!="")
	{
		bv.FindChild("ROIGroupBox").checked=true;
		bv.ExecutePlugin();
		VOIFileName = bv.GetPluginStringParameter("VOIFileName"); // with path removed!
		this.fillVOIList();
	}

	var VOIEd  = bv.FindChild("VOIFileEd");
	VOIEd.text = VOIFileName;
}

scriptObj.onSelectPOIFile = function()
{

	var POIFileName = bv.BrowseFile("Select POI File", "POI files (*.poi)");	
	bv.PrintToLog("Script>  Selected POI File: " + POIFileName);	
	bv.SetPluginStringParameter("POIFileName", POIFileName);
	bv.SetPluginStringParameter("Command", "SelectPOIFile");

	if(POIFileName == "")
	{
		bv.FindChild("ROIGroupBox").checked=false;
		bv.SetPluginStringParameter("POIFileName", "");
		bv.FindChild("ROIsLsWdg").clear();
	}
	if (POIFileName!="")
	{
		bv.FindChild("ROIGroupBox").checked=true;
		bv.ExecutePlugin();
		POIFileName = bv.GetPluginStringParameter("POIFileName"); // with path removed!
		this.fillPOIList();
	}

	var POIEd  = bv.FindChild("POIFileEd");
	POIEd.text = POIFileName; 

}

scriptObj.fillVOIList = function()
{	
	var dlg = this.PluginDialog; // less to write
	var ROIsLW = bv.FindChild("ROIsLsWdg");
	var ROIEd  = bv.FindChild("VOIFileEd");

	var NROIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");
	ROIsLW.clear();

	for(var i=0; i<NROIs; i++)
	{
		var ROINameID = "NameOfROI-" + (i+1);
		var ROIName = bv.GetPluginStringParameter(ROINameID);
		ROIsLW.addItem(ROIName);
	}
	if(NROIs > 0)
		ROIsLW.currentRow = 0;
}

scriptObj.fillPOIList = function()
{	
	var dlg = this.PluginDialog; // less to write
	var ROIsLW = bv.FindChild("ROIsLsWdg");
	var ROIEd  = bv.FindChild("POIFileEd");

	var NROIs = bv.GetPluginIntParameter("NrOfPOIsAvailable");
	ROIsLW.clear();

	for(var i=0; i<NROIs; i++)
	{
		var ROINameID = "NameOfROI-" + (i+1);
		var ROIName = bv.GetPluginStringParameter(ROINameID);
		ROIsLW.addItem(ROIName);
	}
	if(NROIs > 0)
		ROIsLW.currentRow = 0;
}

scriptObj.fillClassTable = function()
{

	var ClassTableWidget = bv.FindChild("ClassTableWidget");
	//ClassTableWidget.clear();

	var NrOfClasses = bv.GetPluginIntParameter("NrOfClasses");
	ClassTableWidget.rowCount = NrOfClasses;

	bv.PrintToLog("// Script> Number of classes: "+NrOfClasses);

	for (var i=0; i<NrOfClasses; i++)
	{
		var ClassNameStr = "NameOfClass-" + (i+1);
		var ClassName = bv.GetPluginStringParameter(ClassNameStr);
		bv.PrintToLog("// Script> Class n. "+(i+1)+": "+ClassName);

		//var itemNr = new QTableWidgetItem;
		//itemNr.setText(i+1); 
		//ClassTableWidget.setItem(i, 0, itemNr);
		//ClassTableWidget.setColumnWidth (0, 50);

		var itemName = new QTableWidgetItem;
		itemName.setText(ClassName); 
		ClassTableWidget.setItem(i, 0, itemName);
		ClassTableWidget.setColumnWidth (0, 100);

	    	var itemSelect = new QTableWidgetItem;
		itemSelect.setCheckState(Qt.Checked);  // itemSelect.setCheckState(Qt.Unchecked);
	    	ClassTableWidget.setItem(i, 1, itemSelect);
          	ClassTableWidget.setColumnWidth (1, 100);

	}
	
}

scriptObj.fillStudyTable = function()
{

	var StudyTableWidget = bv.FindChild("StudyTableWidget");
	//ClassTableWidget.clear();

	var NrOfStudies = bv.GetPluginIntParameter("NrOfStudies");
	StudyTableWidget.rowCount = NrOfStudies;
	//ClassTableWidget.columnCount = 3;

	bv.PrintToLog("// Script> Number of studies: "+NrOfStudies);

	for (var i=0; i<NrOfStudies; i++)
	{
		var StudyNameStr = "NameOfStudy-" + (i+1);
		var StudyName = bv.GetPluginStringParameter(StudyNameStr);
		bv.PrintToLog("// Script> Study n. "+(i+1)+": "+StudyName);

		//var itemNr = new QTableWidgetItem;
		//itemNr.setText(i+1); 
		//ClassTableWidget.setItem(i, 0, itemNr);
		//ClassTableWidget.setColumnWidth (0, 50);

		var itemName = new QTableWidgetItem;
		itemName.setText(StudyName); 
		StudyTableWidget.setItem(i, 0, itemName);
		StudyTableWidget.setColumnWidth (0, 100);

	    	var itemGroup = new QTableWidgetItem;
		itemGroup.setText("1");
		StudyTableWidget.setItem(i, 1, itemGroup);
		StudyTableWidget.setColumnWidth (1, 100);

		//bv.PrintToLog("// Script : " + StudyTableWidget.item(i,1).text());

	}
}

scriptObj.onSelectVMPFile = function()
{
	var VMPFileName = bv.BrowseFile("Select VMP File", "VMP files (*.vmp)");	
	bv.PrintToLog("Script>  Selected VMP File: " + VMPFileName);	
	bv.SetPluginStringParameter("VMPFileName", VMPFileName);
	bv.SetPluginStringParameter("Command", "SelectMAPFile");
	if (VMPFileName!="")
	{
		bv.ExecutePlugin();
		VMPFileName = bv.GetPluginStringParameter("VMPFileName"); // with path removed!
	}
	var VMPEd  = bv.FindChild("VMPFileEd");
	VMPEd.text = VMPFileName;

	bv.PrintToLog("// Script> Filling the class table...");
	this.fillClassTable();
	bv.PrintToLog("// Script> Filling the study table...");
	this.fillStudyTable();

	// Choosing the current tab:
	//var MDSSTabWidget = bv.FindChild("MDSSTabWidget");
	//MDSSTabWidget.currentIndex=1;

}

scriptObj.onSelectSMPFile = function()
{
	var SMPFileName = bv.BrowseFile("Select SMP File", "SMP files (*.smp)");	
	bv.PrintToLog("Script>  Selected SMP File: " + SMPFileName);	
	bv.SetPluginStringParameter("SMPFileName", SMPFileName);
	bv.SetPluginStringParameter("Command", "SelectMAPFile");
	if (SMPFileName!="")
	{
		bv.ExecutePlugin();
		SMPFileName = bv.GetPluginStringParameter("SMPFileName"); // with path removed!
	}
	var SMPEd  = bv.FindChild("SMPFileEd");
	SMPEd.text = SMPFileName;

	bv.PrintToLog("// Script> Filling the class table...");
	this.fillClassTable();
	bv.PrintToLog("// Script> Filling the study table...");
	this.fillStudyTable();

	// Choosing the current tab:
	//var MDSSTabWidget = bv.FindChild("MDSSTabWidget");
	//MDSSTabWidget.currentIndex=1;

}

scriptObj.onSelectMSKFile = function()
{
	var MSKFileName = bv.BrowseFile("Select MSK File", "MSK files (*.msk)");	
	bv.PrintToLog("Script>  Selected MSK File: " + MSKFileName);	
	var MSKEd  = bv.FindChild("MSKFileEd");
	if (MSKFileName!="")
	{
		MSKEd.text = MSKFileName;
		bv.FindChild("MSKGroupBox").checked=true;
	}
}

scriptObj.onSelectAllClasses = function()
{

	var ClassTableWidget = bv.FindChild("ClassTableWidget");
	var NrOfClasses = ClassTableWidget.rowCount;

	for (var i=0; i<NrOfClasses; i++)
	{
		ClassTableWidget.item(i,1).setCheckState(Qt.Checked);
	}

}

scriptObj.onDeselectAllClasses = function()
{

	var ClassTableWidget = bv.FindChild("ClassTableWidget");
	var NrOfClasses = ClassTableWidget.rowCount;

	for (var i=0; i<NrOfClasses; i++)
	{
		ClassTableWidget.item(i,1).setCheckState(Qt.Unchecked);
	}

}

scriptObj.onClickedMMDSProj = function()
{
	bv.FindChild("CCAProjGrpBx").setChecked(false);
	bv.FindChild("SAMProjGrpBx").setChecked(false);
}

scriptObj.onClickedCCAProj = function()
{
	bv.FindChild("MMDSProjGrpBx").setChecked(false);
	bv.FindChild("SAMProjGrpBx").setChecked(false);
}

scriptObj.onClickedSAMProj = function()
{
	bv.FindChild("MMDSProjGrpBx").setChecked(false);
	bv.FindChild("CCAProjGrpBx").setChecked(false);
}

// It is important to return script object containing properties and functions to be called from BrainVoyager QX
// 
returnScriptObj = function()
{
	return scriptObj;
}
returnScriptObj(); // we call this at the end to return created script object to BrainVoyager QX
