var bv = BrainVoyager; // use short symbol to access BrainVoyager API
var scriptObj = new Object;

scriptObj.dialogFileName   = "MLRPlugin.ui";  // specify name of file that will be used by BV to create your UI
scriptObj.dialogAccessName = "PluginDialog";         // specify the name you want to use for accessing your UI

scriptObj.initDlg = function()
{
	var dlg = this.PluginDialog; // less to write

	dlg.windowTitle = "Multivariate Linear Regression Analysis"; // use to change dialog title

	// connec UI elements to script funcctions - you need to know the names of your elements (check your definition)
	dlg.quitButton.clicked.connect(this, this.onQuit);
	dlg.goButton.clicked.connect(this, this.onGo);
	
	// First tab (input data)
	bv.FindChild("selectMSKFileBtn").clicked.connect(this, this.onSelectMSKFile);
	bv.FindChild("selectVOIFileBtn").clicked.connect(this, this.onSelectVOIFile);
	bv.FindChild("addVTCBtn").clicked.connect(this, this.onAddVTCFile);
	bv.FindChild("clearVTCsBtn").clicked.connect(this, this.onClearVTCs);
	bv.FindChild("addMDMbtn").clicked.connect(this, this.onLoadVTCsFromMDM);
	
	// Second tab (cross-validation)
	bv.FindChild("NFoldCrossValGrpBx").clicked.connect(this, this.onClickedNFoldCrossVal);
	bv.FindChild("LeaveNRunsOutGrpBx").clicked.connect(this, this.onClickedLeaveNRunsOutCrossVal);
	bv.FindChild("ProtocolConditionsGrpBx").clicked.connect(this, this.onClickedProtocolConditionsCrossVal);

	// Third tab (kernel method)
	bv.FindChild("LSKernelGrpBx").clicked.connect(this, this.onClickedLSKernel);
	bv.FindChild("RRKernelGrpBx").clicked.connect(this, this.onClickedRRKernel);
	bv.FindChild("RRGCVKernelGrpBx").clicked.connect(this, this.onClickedRRGCVKernel);
	bv.FindChild("RVMKernelGrpBx").clicked.connect(this, this.onClickedRVMKernel);
	//bv.FindChild("GPKernelGrpBx").clicked.connect(this, this.onClickedGPKernel);

	// Fourth tab (output)
	// ????
	
	// Check on the current VOIs ...
	var VOIFileName = bv.VOIFileName;
	if(VOIFileName == "")
	{
		bv.FindChild("VOIsLsWdg").clear();
		return;
	}
	bv.SetPluginStringParameter("VOIFileName", VOIFileName); // if there is a VOI file, load and show VOI names in ListWidget
	bv.SetPluginStringParameter("Command", "StartCheck");
	bv.ExecutePlugin();
	this.fillVOIList();
	bv.FindChild("VOIGroupBox").checked=true;
	bv.FindChild("VOIFileEd").text = VOIFileName;
	
}

scriptObj.onQuit = function()
{
	this.PluginDialog.close();
}

scriptObj.onGo = function()
{
	var dlg = this.PluginDialog; // less to write
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// This part is for VOI-based access ...
	var VOIsLW = bv.FindChild("VOIsLsWdg");	
	var n_vois = VOIsLW.count;
	if(n_vois == 0)
	{
		bv.PrintToLog("// Script> No VOIs currently loaded.");
	}
	// get selected VOI:
	var sel_voi = VOIsLW.currentRow;
	if(sel_voi < 0)
	{
		bv.PrintToLog("// Script> No VOIs currently selected.");
	}
	bv.SetPluginIntParameter("SelectedVOIIndex", sel_voi); // tell plugin which VOI has been selected
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////

	var VTCsLW = bv.FindChild("VTCsLsWdg");
	var SDMsLW = bv.FindChild("SDMsLsWdg");
	
	var n_vtcs = VTCsLW.count;
	if(n_vtcs == 0)
	{
		bv.MessageBox("No VTC files available!")
		return;
	}

	var n_sdms = SDMsLW.count;
	if(n_sdms != n_vtcs)
	{
		bv.MessageBox("SDM files missing!")
		return;
	}

	bv.SetPluginIntParameter("UseMask", bv.FindChild("MSKGroupBox").checked);
	bv.SetPluginIntParameter("UseVOIs", bv.FindChild("VOIGroupBox").checked);

	var MSKEd  = bv.FindChild("MSKFileEd");
	var MSKFileName = MSKEd.text; 
	if(MSKFileName == "")
	{
		//bv.MessageBox("This plugin needs a mask file to be specified!");
		//return;
	}

	var VOIEd  = bv.FindChild("VOIFileEd");
	var VOIFileName = VOIEd.text; 
	if(VOIFileName == "")
	{
		//bv.MessageBox("This plugin needs a mask file to be specified!");
		//return;
	}

	var MLKREd  = bv.FindChild("MLKRFileEd");
	var MLKRFileName = MLKREd.text; 
	if(MLKRFileName == "")
	{
		bv.MessageBox("Please specify a name for the output MLKR file!");
		return;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Now we need to pass all parameters ...

	// VTC and SDM files ...	
	bv.SetPluginIntParameter("NrOfVTCs", n_vtcs);
	for(var i=0; i<n_vtcs; i++)
	{
		var VTCFileName = VTCsLW.item(i).text();
		var VTCFileId = "VTCFile" + i + "Name";
		//bv.PrintToLog("Script> Selected VTC File: " + VTCFileId);
		bv.SetPluginStringParameter(VTCFileId, VTCFileName);
		var SDMFileName = SDMsLW.item(i).text();
		var SDMFileId = "SDMFile" + i + "Name";
		//bv.PrintToLog("Script> Selected SDM File: " + SDMFileId);
		bv.SetPluginStringParameter(SDMFileId, SDMFileName);
	}

	// Mask file
	bv.SetPluginStringParameter("MSKFileName", MSKFileName);
	bv.PrintToLog("Script> Selected MSK File: " + MSKFileName);

	// VOI file
	bv.SetPluginStringParameter("VOIFileName", VOIFileName);
	bv.PrintToLog("Script> Selected VOI File: " + VOIFileName);

	// Results file
	bv.SetPluginStringParameter("MLKRFileName", MLKRFileName);
	bv.PrintToLog("Script> Output MLKR File: " + MLKRFileName);
	
	// Settings from Tab 2 and Tab 3
	if(bv.FindChild("NFoldCrossValGrpBx").checked)
	{
		bv.SetPluginIntParameter("CrossValidationMethod", 1); // n-fold cross-validation (no protocol)
		var NrOfFolds = bv.FindChild("FoldsSpBx").value;
		bv.SetPluginIntParameter("NrOfFolds", NrOfFolds);
		bv.PrintToLog("Script> Cross-validation method: N-fold cross-validation. Nr. of folds = " + NrOfFolds );
	}
	if(bv.FindChild("LeaveNRunsOutGrpBx").checked)
	{
		bv.SetPluginIntParameter("CrossValidationMethod", 2); // leave n runs out (no protocol)
		var NrOfRuns = bv.FindChild("NRunsOutSpBx").value;
		bv.PrintToLog("Script> Cross-validation method: Leave N runs out. Nr. of runs out = " + NrOfRuns );
		//return;
	}
	if(bv.FindChild("ProtocolConditionsGrpBx").checked)
	{
		bv.SetPluginIntParameter("CrossValidationMethod", 3); // based on protocol conditions
		bv.PrintToLog("Script> Protocol-based cross-validation not implemented yet. Exiting plugin.");
		// set all settings to plugin parameters ...
		return;
	}

	// Kernel methods
	// Defaults:
	bv.SetPluginFloatParameter("LogLambda", 0.0);
	bv.SetPluginFloatParameter("LogMinLambda", 1.0);
	bv.SetPluginFloatParameter("LogStpLambda", 0.5);
	bv.SetPluginIntParameter("NrOfLambdaPoints", 17);

	var Opt_KernelScale = bv.FindChild("KernelScaleFcnCmbBx").currentIndex;
	var Str_KernelScale = bv.FindChild("KernelScaleFcnCmbBx").currentText;
	bv.PrintToLog("Script> Kernel downscaling : " + Str_KernelScale);
	bv.SetPluginIntParameter("KernelScale", Opt_KernelScale);

	bv.SetPluginIntParameter("Algorithm", 0); // Algorithm not (yet) selected
	if(bv.FindChild("LSKernelGrpBx").checked)
	{
		bv.SetPluginIntParameter("Algorithm", 1); // least squares
		bv.PrintToLog("Script> Kernel method: Least squares");
		//return;
	}
	
	if(bv.FindChild("RRKernelGrpBx").checked)
	{
		bv.SetPluginIntParameter("Algorithm", 2); // ridge regression
		bv.PrintToLog("Script> Kernel method: Ridge regression");
		var Lambda = bv.FindChild("LambdaDSpBx").value;
		bv.SetPluginFloatParameter("LogLambda", Lambda);
		//return;
	}

	if(bv.FindChild("RRGCVKernelGrpBx").checked)
	{
		bv.SetPluginIntParameter("Algorithm", 3); // ridge regression with generalized cross-validation
		bv.PrintToLog("Script> Kernel method: Ridge regression with generalized cross-validation");
		var MinLambda = bv.FindChild("MinLambdaDSpBx").value;
		bv.SetPluginFloatParameter("LogMinLambda", MinLambda);
		var StpLambda = bv.FindChild("StpLambdaDSpBx").value;
		bv.SetPluginFloatParameter("LogStpLambda", StpLambda);
		var NrLambda = bv.FindChild("NLambdaSpBx").value;
		bv.SetPluginIntParameter("NrOfLambdaPoints", NrLambda);
		//return;
	}

	if(bv.FindChild("RVMKernelGrpBx").checked)
	{
		bv.SetPluginIntParameter("Algorithm", 4); // ridge regression
		bv.PrintToLog("Script> Kernel method: Relevance vector machine");
		var MaxIts = bv.FindChild("MaxItsSpBx").value;
		bv.PrintToLog("Script> RVM method: Max nr. of iterations = " + MaxIts );
		bv.SetPluginIntParameter("MaxNrOfIterations", MaxIts);
		var eD = bv.FindChild("MinDeltaSpBx").value;
		bv.PrintToLog("Script> RVM method: MIN_DELTA_LOGALPHA = 1e" + eD );
		bv.SetPluginIntParameter("eMIN_DELTA_LOGALPHA", eD);
		var eA = bv.FindChild("MaxAlphaSpBx").value;
		bv.PrintToLog("Script> RVM method: ALPHA_MAX = 1e" + eA );
		bv.SetPluginIntParameter("eALPHA_MAX", eA);
		//bv.PrintToLog("Script> RVM not implemented yet. Exiting plugin.");
		//return;
	}

	//if(bv.FindChild("GPKernelGrpBx").checked)
	//{
		//bv.SetPluginIntParameter("Algorithm", 4); // ridge regression
		//bv.PrintToLog("Script> Kernel method: Gassian Process");
		//return;
	//}


	// Settings from Tab 4 (options)
	// Predictors
	var ExcNPred = bv.FindChild("ExcNPredSpBx").value;
	bv.SetPluginIntParameter("ExcNPred",ExcNPred);

	if (bv.FindChild("NoPredScaleRadioBtn").checked) 
	{
		bv.SetPluginIntParameter("PredScaleType", 0); // no scaling
	}
	if (bv.FindChild("ZtrPredScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("PredScaleType", 1); // z-transform
	}
	if (bv.FindChild("PredScale01RadioBtn").checked)
	{
		bv.SetPluginIntParameter("PredScaleType", 2); // scale between 0 and 1
	}
	if (bv.FindChild("PredScale11RadioBtn").checked)
	{
		bv.SetPluginIntParameter("PredScaleType", 3); // scale between -1 and 1
	}

	// Time course normalization
	var TCScaleFactor = bv.FindChild("TCScaleFactorSpBx").value;
	bv.SetPluginIntParameter("TCScaleFactor", TCScaleFactor);
	bv.PrintToLog("Script> Applying a scale factor to time-series of 10^: "+TCScaleFactor);
	if (bv.FindChild("noTCNormRadioBtn").checked)
	{
		bv.SetPluginIntParameter("TCNormType", 0);
	}
	if (bv.FindChild("ztrTCNormRadioBtn").checked)
	{
		bv.SetPluginIntParameter("TCNormType", 1);
	}
	if (bv.FindChild("pscTCNormRadioBtn").checked)
	{
		bv.SetPluginIntParameter("TCNormType", 2);
	}

	// Map scaling
	if (bv.FindChild("MapNoScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutMapNormType", 0);
	}
	if (bv.FindChild("MapZtrScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutMapNormType", 1);
	}
	if (bv.FindChild("MapPSCScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutMapNormType", 2);
	}

	// Prediction scaling
	if (bv.FindChild("PredNoScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutPredNormType", 0);
	}
	if (bv.FindChild("PredZtrScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutPredNormType", 1);
	}
	if (bv.FindChild("PredPSCScaleRadioBtn").checked)
	{
		bv.SetPluginIntParameter("OutPredNormType", 2);
	}

	bv.SetPluginStringParameter("Command", "Go");
	this.PluginDialog.close();
	bv.ExecutePlugin();

	var docVMR = bv.ActiveDocument;	
	docVMR.ShowVolumeMapsDlg();

}

scriptObj.onSelectVOIFile = function()
{
	var VOIFileName = bv.BrowseFile("Select VOI File", "VOI files (*.voi)");
	bv.PrintToLog("Script>  Selected VOI File: " + VOIFileName);	
	var VOIEd  = bv.FindChild("VOIFileEd");
	VOIEd.text = VOIFileName; 
		
	//var dlg = this.PluginDialog; // less to write
		
	if(VOIFileName == "")
	{
		bv.FindChild("VOIsLsWdg").clear();
		return;
	}
		
	// let plugin load VOI file:
	bv.SetPluginStringParameter("VOIFileName", VOIFileName);
	bv.SetPluginStringParameter("Command", "LoadVOIFile"); // use special command, e.g "LoadVOIFile"
	bv.ExecutePlugin();
	this.fillVOIList();
	bv.FindChild("VOIGroupBox").checked=true;

}

scriptObj.fillVOIList = function()
{	
	var dlg = this.PluginDialog; // less to write
	var VOIsLW = bv.FindChild("VOIsLsWdg");
	//var VOIEd  = bv.FindChild("VOIFileEd");

	var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");

	//VOIEd.text = bv.VOIFileName;
	VOIsLW.clear();

	for(var i=0; i<NVOIs; i++)
	{
		var VOINameID = "NameOfVOI-" + (i+1);
		var VOIName = bv.GetPluginStringParameter(VOINameID);
		VOIsLW.addItem(VOIName);
	}
	if(NVOIs > 0)
		VOIsLW.currentRow = 0;
}

scriptObj.onLoadVTCsFromMDM = function()
{
	var MDMFileName = bv.BrowseFile("Select MDM File", "MDM files (*.mdm)");
	
	var VTCList = new Array();
	VTCList = bv.GetVTCsOfMDM(MDMFileName);
	var SDMList = new Array();
	//SDMList = bv.GetSDMsOfMDM(MDMFileName); // NOT AVAILABLE

	var VTCsLW  = bv.FindChild("VTCsLsWdg");
	var SDMsLW  = bv.FindChild("SDMsLsWdg");
	
	VTCsLW.clear();
	VTCsLW.addItems(VTCList);
	
	SDMsLW.clear();
	//SDMsLW.addItems(SDMList);
	
	var found = 0;
	var n_vtcs = VTCsLW.count;
	for(var i=0; i<n_vtcs; i++)
	{

		var VTCFileName = VTCsLW.item(i).text();
		bv.SetPluginStringParameter("VTCFile", VTCFileName);
		bv.SetPluginStringParameter("Command", "CheckAddedVTC");
		var ret = bv.ExecutePlugin();

		if(!ret)
		{
			bv.MessageBox("Error selecting this VTC: Inexistent file!");
			return false;
		}

		var SDMFileName = bv.BrowseFile("Select SDM File for VTC file: "+VTCFileName, "SDM files (*.sdm)");	
		if(SDMFileName == "")
		{
			VTCsLW.clear();
			SDMsLW.clear();
			bv.PrintToLog("Script> MDM entry not completed. Exiting plugin.");
			return;
		}
		bv.PrintToLog("Script>  Selected SDM File: " + SDMFileName);
		SDMsLW.addItem(SDMFileName);

	}

	if(bv.FindChild("ProtocolConditionsGrpBx").checked)
	{
		// Here we should populate the condition list based on the current list of VTCs
		this.checkProtocolsOfAllVTCs();
	}

	//bv.PrintToLog("Script> MDM Button not implemented yet. Exiting plugin.");
	//return;

}

scriptObj.onAddVTCFile = function()
{
	var VTCFileName = bv.BrowseFile("Select VTC File", "VTC files (*.vtc)");
	if(VTCFileName == "")
		return;
	bv.PrintToLog("Script>  Selected VTC File: " + VTCFileName);	
	var SDMFileName = bv.BrowseFile("Select SDM File", "SDM files (*.sdm)");	
	if(SDMFileName == "")
		return;
	bv.PrintToLog("Script>  Selected SDM File: " + SDMFileName);

	////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////
	var VTCsLW  = bv.FindChild("VTCsLsWdg");
	var SDMsLW  = bv.FindChild("SDMsLsWdg");

	var found = 0;
	var n_vtcs = VTCsLW.count;
	for(var i=0; i<n_vtcs; i++)
	{
		var VTCPathFile = VTCsLW.item(i).text();
		if ( VTCPathFile == VTCFileName)
		{
			bv.MessageBox("Error selecting this VTC: Duplicate file!");
			return false;
		}

		bv.SetPluginStringParameter("VTCFile", VTCPathFile);
		bv.SetPluginStringParameter("Command", "CheckAddedVTC");
		var ret = bv.ExecutePlugin();

		if(!ret)
		{
			bv.MessageBox("Error selecting this VTC: Inexistent file!");
			return false;
		}

	}
	////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////

	VTCsLW.addItem(VTCFileName);	
	SDMsLW.addItem(SDMFileName);

	if(bv.FindChild("ProtocolConditionsGrpBx").checked)
	{
		// Here we should populate the condition list based on the current list of VTCs
		this.checkProtocolsOfAllVTCs();
	}

}

scriptObj.onSelectMSKFile = function()
{
	var MSKFileName = bv.BrowseFile("Select MSK File", "MSK files (*.msk)");	
	bv.PrintToLog("Script>  Selected MSK File: " + MSKFileName);	
	var MSKEd  = bv.FindChild("MSKFileEd");
	MSKEd.text = MSKFileName; 
	if(MSKFileName == "")
	{
		return;
	}
	bv.FindChild("MSKGroupBox").checked=true;
}

scriptObj.onClearVTCs = function()
{
	bv.FindChild("VTCsLsWdg").clear();
	bv.FindChild("SDMsLsWdg").clear();
	bv.FindChild("ConditionsLsWdg").clear();
	bv.FindChild("UsePRTCmBx").clear();
}

scriptObj.onClickedNFoldCrossVal = function()
{
	bv.FindChild("LeaveNRunsOutGrpBx").setChecked(false);
	bv.FindChild("ProtocolConditionsGrpBx").setChecked(false);
}

scriptObj.onClickedLeaveNRunsOutCrossVal = function()
{
	bv.FindChild("NFoldCrossValGrpBx").setChecked(false);
	bv.FindChild("ProtocolConditionsGrpBx").setChecked(false);
}

scriptObj.onClickedProtocolConditionsCrossVal = function()
{
	bv.FindChild("NFoldCrossValGrpBx").setChecked(false);
	bv.FindChild("LeaveNRunsOutGrpBx").setChecked(false);
	
	// Here we should populate the condition list based on the current list of VTCs
	this.checkProtocolsOfAllVTCs();
}

scriptObj.onClickedLSKernel = function()
{
	//bv.FindChild("LSKernelGrpBx").setChecked(false);
	bv.FindChild("RRKernelGrpBx").setChecked(false);
	bv.FindChild("RRGCVKernelGrpBx").setChecked(false);
	bv.FindChild("RVMKernelGrpBx").setChecked(false);
	//bv.FindChild("GPKernelGrpBx").setChecked(false);
	if(bv.FindChild("LSKernelGrpBx").checked)
	{
		bv.FindChild("KernelScaleFcnCmbBx").setCurrentIndex(0);
	}
}

scriptObj.onClickedRRKernel = function()
{
	bv.FindChild("LSKernelGrpBx").setChecked(false);
	//bv.FindChild("RRKernelGrpBx").setChecked(false);
	bv.FindChild("RRGCVKernelGrpBx").setChecked(false);
	bv.FindChild("RVMKernelGrpBx").setChecked(false);
	//bv.FindChild("GPKernelGrpBx").setChecked(false);
	if(bv.FindChild("RRKernelGrpBx").checked)
	{
		bv.FindChild("KernelScaleFcnCmbBx").setCurrentIndex(0);
	}
}

scriptObj.onClickedRRGCVKernel = function()
{
	bv.FindChild("LSKernelGrpBx").setChecked(false);
	bv.FindChild("RRKernelGrpBx").setChecked(false);
	//bv.FindChild("RRGCVKernelGrpBx").setChecked(false);
	bv.FindChild("RVMKernelGrpBx").setChecked(false);
	//bv.FindChild("GPKernelGrpBx").setChecked(false);
	if(bv.FindChild("RRGCVKernelGrpBx").checked)
	{
		bv.FindChild("KernelScaleFcnCmbBx").setCurrentIndex(0);
	}
}

scriptObj.onClickedRVMKernel = function()
{
	bv.FindChild("LSKernelGrpBx").setChecked(false);
	bv.FindChild("RRKernelGrpBx").setChecked(false);
	bv.FindChild("RRGCVKernelGrpBx").setChecked(false);
	//bv.FindChild("RVMKernelGrpBx").setChecked(false);
	//bv.FindChild("GPKernelGrpBx").setChecked(false);
	if(bv.FindChild("RVMKernelGrpBx").checked)
	{
		bv.FindChild("KernelScaleFcnCmbBx").setCurrentIndex(2);
	}
}

scriptObj.onClickedGPKernel = function()
{
	bv.FindChild("LSKernelGrpBx").setChecked(false);
	bv.FindChild("RRKernelGrpBx").setChecked(false);
	bv.FindChild("RRGCVKernelGrpBx").setChecked(false);
	bv.FindChild("RVMKernelGrpBx").setChecked(false);
	//bv.PrintToLog("-----HERE-----");
	//bv.FindChild("GPKernelGrpBx").setChecked(false);
}

scriptObj.onClickedMSKOpt = function()
{
	bv.FindChild("VOIGroupBox").setChecked(false);
}

scriptObj.onClickedVOIOpt = function()
{
	bv.FindChild("MSKGroupBox").setChecked(false);
}

scriptObj.checkProtocolsOfAllVTCs = function()
{
	var VTCsLW  = bv.FindChild("VTCsLsWdg");
	var SDMsLW  = bv.FindChild("SDMsLsWdg");
	var CondsLW = bv.FindChild("ConditionsLsWdg");
	//var BaseCB = bv.FindChild("UsePRTCmBx");

	CondsLW.clear();
	BaseCB.clear();
	
	var n_vtcs = VTCsLW.count;
	if(n_vtcs == 0)
	{
		//bv.MessageBox("No VTC files available!");
		return false;
	}

	//var n_conds = CondsLW.count;
	//if(n_conds == 0)
	//{
		//bv.MessageBox("No protocol conditions available!");
		//return false;
	//}
	//var sel_cond_idx  = CondsLW.currentRow;
	//var sel_cond_name = CondsLW.currentItem().text();
	
	// check that all VTCs have a protocol with same condition as selected one for first VTC
	for(var i=0; i<n_vtcs; i++)
	{
		var VTCPathFile = VTCsLW.item(i).text();
		// let plugin provide conditions of VTC's protocol 
		bv.SetPluginStringParameter("VTCFile", VTCPathFile);
		bv.SetPluginStringParameter("Command", "GetCondsOfVTCProtocol");
		var ret = bv.ExecutePlugin();
		if(!ret)
		{
			bv.MessageBox("Error getting conditions from VTCs protocol - protocol attached to all VTCs?");
			return false;
		}
		
		var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
		//if(NrOfConditions < n_conds)
			//return false;

		if (i==0) // for the first VTC use all conditions
		{
			for (var j=0; j<NrOfConditions; j++)
			{
				var CondNameID = "Cond" + j;
				var CondName = bv.GetPluginStringParameter(CondNameID);
				CondsLW.addItem(CondName);
				//BaseCB.addItem(CondName);			
			}
		}

		var n_conds = CondsLW.count; 
		if (i>=1) // for all other VTCs checks that the same conditions in the same order are present in the protocol ...
		{
			for (var j=0; j<n_conds; j++)
			{

				var found = 0;

				for (var k=0; k<NrOfConditions; k++)
				{
						var CondNameID = "Cond" + k;
						var CondName = bv.GetPluginStringParameter(CondNameID);

						bv.PrintToLog("Script> Comparing: " + CondName + " with: " + CondsLW.item(j).text()); 

						if ( CondName == CondsLW.item(j).text())
							found = 1;
				}
				
				if (found==0)
				{
					bv.MessageBox("Incompatible PRT file for VTC: "+ VTCPathFile + " - Please check the protocol linked to this VTC!");
					VTCsLW.clear();
					SDMsLW.clear();
					CondsLW.clear();
					//BaseCB.clear();
					return false;
				}

			}
		}

	}

	return true;
}

// It is important to return script object containing properties and functions to be called from BrainVoyager QX
// 
returnScriptObj = function()
{
	return scriptObj;
}
returnScriptObj(); // we call this at the end to return created script object to BrainVoyager QX
