// nifticonverter.js: script to manage the user interface nifticonverter.ui and to communicate with the NIfTI converter plugin.
// This bundle consists of *.js, *.ui and *.dylib/*.dll/*.so files, place in /Documents/BVQXExtensions/Plugins_32 or 64 (Mac)/ folder.
// Functionality: Converts between NIfTI and BrainVoyager file formats. 
// Usage: In the BrainVoyager menu, select 'Plugins' > NIfTI converter
// Hester Breman, 2003-2011; current version for NIfTI converter v1.08.220811.

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DECLARATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "NIfTI converter"; 
scriptObj.dialogFileName 		= "nifticonverter.ui"; 
scriptObj.dialogAccessName      = "niftiDlg"; 

// global variables
var command = "Unknown";
var incompleteinfo = false;
var hasposfile = false;
var hasniftifile = false;
var hastrffile = false;
var openInBVQX = false;
var doBatch = false;

// file type IDs correspond to plugin converter's IDs
const FILE_TYPE_UNKNOWN = 0;
const NIFTI_FILE_TYPE = 1; 
const NIFTI_PAIR_FILE_TYPE = 2;
const ANALYZE75_FILE_TYPE = 3;
const NIFTI_IMG_TYPE = 4;
const BV_TYPE_VMR = 20;
const BV_TYPE_FMR = 21;
const BV_TYPE_AMR = 22;
const BV_TYPE_VTC = 24;
const BV_TYPE_VOI = 27;
const BV_TYPE_DMR = 29;
const BV_TYPE_VMP = 201;
const BV_TYPE_VMP_AR = 202;
const BV_TYPE_VMP_NR = 203;
const FILE_TYPE_TXT = 401;
const BV_TYPE_MAP = 200;
const BV_TYPE_STC = 23;
const BV_TYPE_DWI = 30;
const BV_TYPE_VDW = 31;
const BV_TYPE_GLM = 205;
const BV_TYPE_AVA = 206;
const BV_TYPE_PRT = 208;
const BV_TYPE_TRF = 301;
const BV_TYPE_POS = 302;
// submenu options
const PRINT_TO_LOG = 1;
const PRINT_TO_FILE = 2;
const PRINT_TO_LOG_AND_FILE = 3;
const PRINT_HIST = 4;
// 	enum ConvMenuOptions
const CONVERT_SINGLE_FILE = 1;
const CONVERT_BATCH = 2;
const CONVERT_REGISTERED = 3;
const CONVERT_REGISTERED_BATCH = 4;
// enum POSMENUOPTIONS
const POS_MATRIX_FROM_FILE = 1;
const POS_ADD_TO_FILE = 2;
const POS_BVQXPOS_FROM_NIIPOS = 3; // not active yet
const POS_COMPUTE_COORDS_OF_POINTS = 4; // not active yet
// enum TRFMENUOPTIONS
const TRF_INVERT_MAT = 1;
const TRF_APPLY_TRF_MAN = 2;
const TRF_APPLY_TRF_FILE = 3;
const TRF_COMPUTE_BBOX = 4;

// ~~~ nicknames for children ~~~
// global tab
var optConvertButton, optInfoButton, optPosButton, optManipButton, optTrfButton, optBatchButton, startButton;
var niftiNameLabel, getNiftiFileButton;
// conversion tab
var niiFromButton, hdrFromButton, vmrFromButton, fmrFromButton, dmrFromButton, vtcFromButton, vmpFromButton, voiFromButton; 
var niiToButton, hdrToButton, vmrToButton, fmrToButton, dmrToButton, vtcToButton, vmpToButton; 
var transformBox, getBatchConvButton, getNIfTIConvButton, transformBox, flipBox, batchBox, batchFileLabel, openInBVQXBox, zeropadBox, intensityBox, vointerpBox, resetBox;
// info tab
var niiPrintButton, hdrPrintButton, vmrPrintButton, fmrPrintButton, dmrPrintButton, vtcPrintButton, vmpPrintButton, voiPrintButton; 
var getniftiInfoButton, destBothButton, destFileButton, destLogButton; 
// positioning tab
var getPosFileButton, createPosFileButton, posToImgButton, printPosMatButton, calcPosButton, p00, p01, p02, p03, p10, p11, p12, p13, p20, p21, p22, p23, p30, p31, p32, p33;
var nxBox, nyBox, nzBox, dxBox, dyBox, dzBox;
// header manipulation tab
var optCopyButton, optFldManipButton; 
var leQformCode, leQb, leQc, leQd, leQoffsetX, leQoffsetY, leQoffsetZ, leSformCode, leSrow00, leSrow01, leSrow02,leSrow03, leSrow11 ,leSrow12,leSrow13, leSrow21, leSrow22, leSrow23;
var rbtnCopyPos,rbtnCopyAll, rbtnNiiType, rbtnHdrType, rbtnNiiTarget, rbtnHdrTarget, lblOtherNIfTIfile, btnOtherNIfTIfile;
// transformation tab
var getNIfTIFileToTrfButton, invertTrfButton,applyManTrfButton,applyBVQXTrfButton, getTrfFileTypeBox, transformGrBox, t00, t01, t02, t03, t10, t11, t12, t13, t20, t21, t22, t23, t30, t31, t32, t33; 
 // batch tab
var writeBatchFileButton;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ GENERIC FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  


scriptObj.errorfree = function() {

    var errornr = BrainVoyager.GetPluginIntParameter("ErrorNr");
    if (errornr == 0) return true;
    else return false;
}

scriptObj.checkerrorstatus = function() {

    var errornr = BrainVoyager.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyager.GetPluginStringParameter("ErrorMsg");
            BrainVoyager.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.start = function() {
    
    var success = true;   
    // check if status is prepared 
    if (optConvertButton.checked == 1) {
        this.prepareConversion();
    } else if (optInfoButton.checked == 1) {   
        success = this.prepareCollectInfo();
    } else if (optPosButton.checked == 1) {    
        this.preparePosFuncs();
    } else if (optManipButton.checked == 1) {
        this.prepareHdrManipFuncs();
    } else if (optTrfButton.checked == 1) {    
        this.prepareTrfFuncs();
    } else if (optBatchButton.checked == 1) { 
        this.prepareBatchFuncs();
    } else {
        BrainVoyager.PrintToLog("Could not find which option was checked.");
        incompleteinfo = true;
    }   
    
    // then start plugin
    if ((!incompleteinfo) && (success)) {
        BrainVoyager.SetPluginStringParameter("Command", "Start");    
        BrainVoyager.ExecutePlugin();
    
        // ask parameters from plugin
        if ((this.errorfree()) && (!doBatch) && (openInBVQX)) {
            var convertedname = BrainVoyager.GetPluginStringParameter("ConvertedName");
            BrainVoyager.PrintToLog("Opening " + convertedname + "...");
            var bvqxdoc = BrainVoyager.OpenDocument(convertedname.toString());
        }
    } else {
        BrainVoyager.PrintToLog("Cannot start function: incomplete information. Please check whether all parameters have been collected.");
    }   
    
    // now close the dialog
    this.niftiDlg.close(); 
}

scriptObj.getNiftiFile = function() {

    var filetypeid = FILE_TYPE_UNKNOWN;
    if (optConvertButton.checked == 1) {
        filetypeid = this.getConvertFromFileId();
    } else if (optInfoButton.checked == 1) {
        filetypeid = this.getInfoFileId();
    } else if (optTrfButton.checked == 1) {    
        filetypeid = this.getFiletypeToTransform();
    } else if (optManipButton.checked == 1) {
        filetypeid = this.getManipulatedFileType();
    } else {
        BrainVoyager.PrintToLog("Source of signal to get NIfTI file not recognised.");
    }
                  
    if (filetypeid != FILE_TYPE_UNKNOWN) {          
        BrainVoyager.SetPluginStringParameter("Command", "GetFile");    
        BrainVoyager.SetPluginIntParameter("FileID", filetypeid);
        BrainVoyager.ExecutePlugin(); 
        var niftifilename = BrainVoyager.GetPluginStringParameter("NiftiFileName");
        niftiNameLabel.text = niftifilename;
        hasniftifile = true;  
                
        // show the parameters in the `Header' tab
        var matrix = this.getMatrixParametersFromPlugin();
        this.setSformMatrix(matrix);
        leQformCode.text = BrainVoyager.GetPluginIntParameter("QFormCode");
        leSformCode.text = BrainVoyager.GetPluginIntParameter("SFormCode");
        leQb.text = BrainVoyager.GetPluginFloatParameter("Qb");
        leQc.text = BrainVoyager.GetPluginFloatParameter("Qc");
        leQd.text = BrainVoyager.GetPluginFloatParameter("Qd");
        leQoffsetX.text = BrainVoyager.GetPluginFloatParameter("QoffsetX");
        leQoffsetY.text = BrainVoyager.GetPluginFloatParameter("QoffsetY");
        leQoffsetZ.text = BrainVoyager.GetPluginFloatParameter("QoffsetZ");
          
    }    
}

scriptObj.isniftifile = function(filetypeid) {
    
    if ((filetypeid == NIFTI_FILE_TYPE) || (filetypeid == NIFTI_PAIR_FILE_TYPE) || (filetypeid == ANALYZE75_FILE_TYPE) ||
        (filetypeid == NIFTI_IMG_TYPE)) return true;
    else return false;
}


scriptObj.printMatrixToLog = function(matrix) {

    BrainVoyager.PrintToLog("Matrix:");
    for (i=0; i<4; i++) {
            BrainVoyager.PrintToLog(matrix[i][0] + ", " + matrix[i][1] + ", " + matrix[i][2] + ", " + matrix[i][3]); 
    }
}

scriptObj.onlyHasNumericValues = function(matrix) {

    var value;
    var hasOnlyNumericValues = true;
    for (i=0; i<4; i++) {
        for (j=0; j<4; j++) {
           if (isNaN(matrix[i][j])) {
                hasOnlyNumericValues = false;
                break;
            }
        }            
    }
    return hasOnlyNumericValues;
}

scriptObj.setMatrixParametersForPlugin = function(matrix) {

    BrainVoyager.SetPluginFloatParameter("m00", matrix[0][0]);
    BrainVoyager.SetPluginFloatParameter("m01", matrix[0][1]);
    BrainVoyager.SetPluginFloatParameter("m02", matrix[0][2]);
    BrainVoyager.SetPluginFloatParameter("m03", matrix[0][3]);

    BrainVoyager.SetPluginFloatParameter("m10", matrix[1][0]);
    BrainVoyager.SetPluginFloatParameter("m11", matrix[1][1]);
    BrainVoyager.SetPluginFloatParameter("m12", matrix[1][2]);
    BrainVoyager.SetPluginFloatParameter("m13", matrix[1][3]);

    BrainVoyager.SetPluginFloatParameter("m20", matrix[2][0]);
    BrainVoyager.SetPluginFloatParameter("m21", matrix[2][1]);
    BrainVoyager.SetPluginFloatParameter("m22", matrix[2][2]);
    BrainVoyager.SetPluginFloatParameter("m23", matrix[2][3]);

    BrainVoyager.SetPluginFloatParameter("m30", matrix[3][0]);
    BrainVoyager.SetPluginFloatParameter("m31", matrix[3][1]);
    BrainVoyager.SetPluginFloatParameter("m32", matrix[3][2]);
    BrainVoyager.SetPluginFloatParameter("m33", matrix[3][3]);
}


scriptObj.getMatrixParametersFromPlugin = function() {

    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = BrainVoyager.GetPluginFloatParameter("m00");
    matrix[0][1] = BrainVoyager.GetPluginFloatParameter("m01");
    matrix[0][2] = BrainVoyager.GetPluginFloatParameter("m02");
    matrix[0][3] = BrainVoyager.GetPluginFloatParameter("m03");
             
    matrix[1][0] = BrainVoyager.GetPluginFloatParameter("m10");
    matrix[1][1] = BrainVoyager.GetPluginFloatParameter("m11");
    matrix[1][2] = BrainVoyager.GetPluginFloatParameter("m12");
    matrix[1][3] = BrainVoyager.GetPluginFloatParameter("m13");
    
    matrix[2][0] = BrainVoyager.GetPluginFloatParameter("m20");
    matrix[2][1] = BrainVoyager.GetPluginFloatParameter("m21");
    matrix[2][2] = BrainVoyager.GetPluginFloatParameter("m22");
    matrix[2][3] = BrainVoyager.GetPluginFloatParameter("m23");
    
    matrix[3][0] = BrainVoyager.GetPluginFloatParameter("m30");
    matrix[3][1] = BrainVoyager.GetPluginFloatParameter("m31");
    matrix[3][2] = BrainVoyager.GetPluginFloatParameter("m32");
    matrix[3][3] = BrainVoyager.GetPluginFloatParameter("m33");        
    
    return matrix;
}
    

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 1. CONVERSIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareConversion = function() {

    // get information from widgets
    var fromID = this.getConvertFromFileId(); 
    var toID = this.getConvertToFileId(); 
    var doTransform = transformBox.checked;
    var doFlip = flipBox.checked;    
    doBatch = batchBox.checked;
    openInBVQX = openinbvqxBox.checked;
    var doZeropad = zeropadBox.checked;
    var adaptIntensity = intensityBox.checked;
    var adaptInterpolation = vointerpBox.checked;
    var resetOrient = resetBox.checked;
        
    // set parameters for plugin // const CONVERT_REGISTERED = 3; // const CONVERT_REGISTERED_BATCH = 4;
    if ((fromID != FILE_TYPE_UNKNOWN) && (toID != FILE_TYPE_UNKNOWN)) { 
        BrainVoyager.SetPluginStringParameter("MenuOption", "Convert"); 
        BrainVoyager.SetPluginIntParameter("SubmenuOption", (doBatch ? CONVERT_BATCH : CONVERT_SINGLE_FILE));
        BrainVoyager.SetPluginIntParameter("ConvertFromID", fromID);
        BrainVoyager.SetPluginIntParameter("ConvertToID", toID);
        BrainVoyager.SetPluginIntParameter("DoFlip", doFlip);
        BrainVoyager.SetPluginIntParameter("DoTransform", doTransform);
        BrainVoyager.SetPluginIntParameter("DoZeropad", doZeropad);
        BrainVoyager.SetPluginIntParameter("OpenInBVQX", openInBVQX);
        BrainVoyager.SetPluginIntParameter("AdaptIntensity", adaptIntensity);
        BrainVoyager.SetPluginIntParameter("AdaptInterpolation", adaptInterpolation);
        BrainVoyager.SetPluginIntParameter("ResetOrientation", resetOrient);
        if (this.isniftifile(fromID)) {
            if (hasniftifile) {
                BrainVoyager.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
            } else {
                BrainVoyager.PrintToLog("Could not find NIfTI file name to convert.");
                incompleteinfo = true;
            }
        }
    }
}

scriptObj.getConvertFromFileId = function() {
    
    var fileid;
    if (niiFromButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrFromButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrFromButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrFromButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrFromButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcFromButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpFromButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else if (voiFromButton.checked == 1) {
       fileid = BV_TYPE_VOI;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}

scriptObj.getConvertToFileId = function() {
  
    var fileid;
    if (niiToButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrToButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrToButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrToButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrToButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcToButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpToButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}

scriptObj.getBatchFile = function() {

    BrainVoyager.SetPluginStringParameter("Command", "GetFile");
    BrainVoyager.SetPluginIntParameter("FileID", FILE_TYPE_TXT);
    BrainVoyager.ExecutePlugin(); 
    var batchfilename = BrainVoyager.GetPluginStringParameter("BatchFileName");
    batchFileLabel.text = batchfilename;
}


scriptObj.enableBatchButton = function() {

    var doBatch = batchBox.checked;
    if (doBatch) getBatchConvButton.enabled = true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 2. INFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareCollectInfo = function() {

    // get information from widgets
    var success = true;
    var submenuoption;
    var fileid = this.getInfoFileId();
    if (destLogButton.checked == 1) {
        submenuoption = PRINT_TO_LOG;
    } else if (destFileButton.checked == 1) {
        submenuoption = PRINT_TO_FILE;
    } else if (destBothButton.checked == 1) {
        submenuoption = PRINT_TO_LOG_AND_FILE;
    } else {
        incompleteinfo = true;
        success = false;
    }
  
    // set parameters for plugin 
    if ((fileid != FILE_TYPE_UNKNOWN) && (!incompleteinfo)) {
    
        BrainVoyager.SetPluginStringParameter("MenuOption", "PrintInfo"); 
        BrainVoyager.SetPluginIntParameter("FileID", fileid);
        BrainVoyager.SetPluginIntParameter("SubmenuOption", submenuoption);
        if (this.isniftifile(fileid)) {
        
            if (hasniftifile) {        
                BrainVoyager.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
            } else {
                incompleteinfo = true;
                success = false;
            }
        }
    } else {
        success = false;
    }
    return success;
}

scriptObj.getInfoFileId = function() {

    var fileid;
    if (niiPrintButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrPrintButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrPrintButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrPrintButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrPrintButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcPrintButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpPrintButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else if (voiPrintButton.checked == 1) {
       fileid = BV_TYPE_VOI;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 3. POSITIONING FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
 
scriptObj.getPosFile = function() {
 
    BrainVoyager.SetPluginStringParameter("Command", "GetFile");
    BrainVoyager.SetPluginIntParameter("FileID", BV_TYPE_POS);
    BrainVoyager.ExecutePlugin(); 
    var posfilename = BrainVoyager.GetPluginStringParameter("PosFileName");
    posFileLabel.text = posfilename;
    hasposfile = true;
}
          
scriptObj.preparePosFuncs = function() {

    // get information from widgets
    var submenuoption, matrix;
    if (printPosMatButton.checked == 1) {
        submenuoption = POS_MATRIX_FROM_FILE;
        if (hasposfile) {
            BrainVoyager.SetPluginStringParameter("PosFileName", posFileLabel.text);
        } else {
            incompleteinfo = true;
        }
    } else if (createPosFileButton.checked == 1) {
        submenuoption = POS_ADD_TO_FILE;
        matrix = this.getPosMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyager.PrintToLog("Cannot proceed: values of positioning matrix are not numeric.");
            incompleteinfo = true;
        } else {
            BrainVoyager.SetPluginIntParameter("nx", nxBox.value);
            BrainVoyager.SetPluginIntParameter("ny", nyBox.value);
            BrainVoyager.SetPluginIntParameter("nz", nzBox.value);                        
            BrainVoyager.SetPluginFloatParameter("dx", dxBox.value);
            BrainVoyager.SetPluginFloatParameter("dy", dyBox.value);
            BrainVoyager.SetPluginFloatParameter("dz", dzBox.value);                        
        }       
    } else if ((submenuoption == POS_MATRIX_FROM_FILE) && (!hasposfile)) {
        BrainVoyager.PrintToLog("Cannot proceed: no positioning file selected."); 
    } else {
        incompleteinfo = true;
    }
    
    // set parameters for plugin
    if (!incompleteinfo) {
        BrainVoyager.SetPluginStringParameter("MenuOption", "PosFuncs"); 
        BrainVoyager.SetPluginIntParameter("SubmenuOption", submenuoption);
        if (submenuoption == POS_ADD_TO_FILE) this.setMatrixParametersForPlugin(matrix);
    }
}

scriptObj.getPosMatrix = function() {

	BrainVoyager.PrintToLog("Get positioning matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = p00.text;
    matrix[0][1] = p01.text;
    matrix[0][2] = p02.text;
    matrix[0][3] = p03.text;
    matrix[1][0] = p10.text;
    matrix[1][1] = p11.text;
    matrix[1][2] = p12.text;
    matrix[1][3] = p13.text;
    matrix[2][0] = p20.text;
    matrix[2][1] = p21.text;
    matrix[2][2] = p22.text;
    matrix[2][3] = p23.text;
    matrix[3][0] = p30.text;
    matrix[3][1] = p31.text;
    matrix[3][2] = p32.text;
    matrix[3][3] = p33.text;    
    return matrix;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 4. HEADER MANIPULATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareHdrManipFuncs = function() {


    var niftitarget = niftiNameLabel.text; // for both submenuoptions    
    if (niftitarget.length != 0) {
           BrainVoyager.SetPluginStringParameter("NiftiFileName", niftitarget); 
           BrainVoyager.SetPluginIntParameter("FileID", this.getManipulatedFileType());
    } else {
            incompleteinfo = true;
    }

    var submenuoption;    
    if (optCopyButton.checked == 1) {
        
        submenuoption = 1;
        var copyall = (rbtnCopyAll.checked == 1);
        BrainVoyager.SetPluginIntParameter("CopyAll", (copyall ? 1 : 0));

        var niftisource = lblOtherNIfTIfile.text;
        if (niftisource.length != 0) {
            BrainVoyager.SetPluginStringParameter("NiftiSourceFileName", niftisource); 
            BrainVoyager.SetPluginIntParameter("SrcFileID", this.getFiletypeToCopy());
        } else {
            incompleteinfo = true;
        }
        
    } else if (optFldManipButton.checked == 1) {
    
        submenuoption = 2;
        matrix = this.getSformMatrix();
        if (this.onlyHasNumericValues(matrix)) {
            this.setMatrixParametersForPlugin(matrix);
        } else {
            incompleteinfo = true; 
        }
        BrainVoyager.SetPluginIntParameter("QFormCode", leQformCode.text);
        BrainVoyager.SetPluginIntParameter("SFormCode", leSformCode.text);
        BrainVoyager.SetPluginFloatParameter("Qb", leQb.text);
        BrainVoyager.SetPluginFloatParameter("Qc", leQc.text);
        BrainVoyager.SetPluginFloatParameter("Qd", leQd.text);
        BrainVoyager.SetPluginFloatParameter("QoffsetX", leQoffsetX.text);
        BrainVoyager.SetPluginFloatParameter("QoffsetY", leQoffsetY.text);
        BrainVoyager.SetPluginFloatParameter("QoffsetZ", leQoffsetZ.text);
        
    }
    
    if (!incompleteinfo) {
    
        BrainVoyager.PrintToLog("Set plugin parameters for header manipulation function...");
        BrainVoyager.SetPluginStringParameter("MenuOption", "HdrFuncs"); 
        BrainVoyager.SetPluginIntParameter("SubmenuOption", submenuoption);
    }
     
    
}

scriptObj.getOtherNiftiFile = function() {

    var filetypeid = this.getFiletypeToCopy();                        
    if (filetypeid != FILE_TYPE_UNKNOWN) {          
        BrainVoyager.SetPluginStringParameter("Command", "GetFile");    
        BrainVoyager.SetPluginIntParameter("FileID", filetypeid);
        BrainVoyager.ExecutePlugin(); 
        var niftifilename = BrainVoyager.GetPluginStringParameter("NiftiFileName");
        lblOtherNIfTIfile.text = niftifilename; 
    }    
}

scriptObj.getSformMatrix = function() {

	BrainVoyager.PrintToLog("Get S-form matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = leSrow00.text;
    matrix[0][1] = leSrow01.text;
    matrix[0][2] = leSrow02.text;
    matrix[0][3] = leSrow03.text;
    matrix[1][0] = leSrow10.text;
    matrix[1][1] = leSrow11.text;
    matrix[1][2] = leSrow12.text;
    matrix[1][3] = leSrow13.text;
    matrix[2][0] = leSrow20.text;
    matrix[2][1] = leSrow21.text;
    matrix[2][2] = leSrow22.text;
    matrix[2][3] = leSrow23.text;
    matrix[3][0] = leSrow30.text;
    matrix[3][1] = leSrow31.text;
    matrix[3][2] = leSrow32.text;
    matrix[3][3] = leSrow33.text;    
    return matrix;
}

scriptObj.setSformMatrix = function(matrix) {

    leSrow00.text = matrix[0][0];
    leSrow01.text = matrix[0][1];
    leSrow02.text = matrix[0][2];
    leSrow03.text = matrix[0][3];    

    leSrow10.text = matrix[1][0];
    leSrow11.text = matrix[1][1];
    leSrow12.text = matrix[1][2];
    leSrow13.text = matrix[1][3]; 
    
    leSrow20.text = matrix[2][0];
    leSrow21.text = matrix[2][1];
    leSrow22.text = matrix[2][2];
    leSrow23.text = matrix[2][3]; 
    
    leSrow30.text = matrix[3][0];
    leSrow31.text = matrix[3][1];
    leSrow32.text = matrix[3][2];
    leSrow33.text = matrix[3][3];             
}

scriptObj.getManipulatedFileType = function() {

    var filetypeid;
    if (rbtnNiiTarget.checked == 1) {
        filetypeid = NIFTI_FILE_TYPE;
    } else if (rbtnHdrTarget.checked == 1) {
        filetypeid = NIFTI_PAIR_FILE_TYPE;
    }
    return filetypeid;
} 

scriptObj.getFiletypeToCopy = function() {
  
    var filetypeid;
    if (rbtnNiiType.checked == 1) {
        filetypeid = NIFTI_FILE_TYPE;
    } else if (rbtnHdrType.checked == 1) {
        filetypeid = NIFTI_PAIR_FILE_TYPE;
    }
    return filetypeid;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 5. TRANSFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

scriptObj.getTrfFile = function() {

    BrainVoyager.SetPluginStringParameter("Command", "GetFile");    
    BrainVoyager.SetPluginIntParameter("FileID", BV_TYPE_TRF);
    BrainVoyager.ExecutePlugin(); 
    var trffilename = BrainVoyager.GetPluginStringParameter("TrfFileName");
    trfFileLabel.text = trffilename;
    hastrffile = true;
}

scriptObj.prepareTrfFuncs = function() {

    // get information from widgets
    var submenuoption, matrix, fileid;
    if (invertTrfButton.checked == 1) {
        submenuoption = TRF_INVERT_MAT;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyager.PrintToLog("Cannot proceed: values of transformation matrix are not numeric.");
            incompleteinfo = true;
        }        
    } else if (applyManTrfButton.checked == 1) {
        submenuoption = TRF_APPLY_TRF_MAN;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyager.PrintToLog("Cannot proceed: values of positioning matrix are not numeric.");
            incompleteinfo = true;
        }        
        fileid = this.getFiletypeToTransform();
    } else if (applyBVQXTrfButton.checked == 1) {
        submenuoption = TRF_APPLY_TRF_FILE;
        fileid = this.getFiletypeToTransform();
        if (hastrffile) {
            BrainVoyager.SetPluginStringParameter("TrfFileName", trfFileLabel.text);
        } else {
            incompleteinfo = true;
            BrainVoyager.PrintToLog("Cannot proceed: no transformation file selected."); 
        }   
    }

    // set parameters for plugin
    if (!incompleteinfo) {
    
        BrainVoyager.PrintToLog("Set plugin parameters for transformation function...");
        BrainVoyager.SetPluginStringParameter("MenuOption", "TrfFuncs"); 
        BrainVoyager.SetPluginIntParameter("SubmenuOption", submenuoption);
        // use manual matrix
        if (submenuoption == 1 || submenuoption == 2) {
            BrainVoyager.PrintToLog("Set matrix fields...");
            this.setMatrixParametersForPlugin(matrix);       
        }
        if (submenuoption == 2 || submenuoption == 3) {
        
        BrainVoyager.PrintToLog("Set file ID: " + fileid);
        
            BrainVoyager.SetPluginIntParameter("FileID", fileid);
            
    /*        if (this.isniftifile(fromID)) {
                if (hasniftifile) {
                    BrainVoyager.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
                    
           BrainVoyager.PrintToLog("Set NIfTI name...");          
                    
                } else {
                    incompleteinfo = true;
                    
           BrainVoyager.PrintToLog("incompleteinfo = true...");

                }
            }  
      */                
        }        
    } else {
        BrainVoyager.PrintToLog("Information not complete.");
    }
}

scriptObj.getTrfMatrix = function() {

	BrainVoyager.PrintToLog("Get transformation matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = t00.text;
    matrix[0][1] = t01.text;
    matrix[0][2] = t02.text;
    matrix[0][3] = t03.text;
    matrix[1][0] = t10.text;
    matrix[1][1] = t11.text;
    matrix[1][2] = t12.text;
    matrix[1][3] = t13.text;
    matrix[2][0] = t20.text;
    matrix[2][1] = t21.text;
    matrix[2][2] = t22.text;
    matrix[2][3] = t23.text;
    matrix[3][0] = t30.text;
    matrix[3][1] = t31.text;
    matrix[3][2] = t32.text;
    matrix[3][3] = t33.text;    
    return matrix;
}


scriptObj.getFiletypeToTransform = function() {
  
    var filetypeid;
    var currentindex = getTrfFileTypeBox.currentIndex;
    switch (currentindex) {
        case 0: filetypeid = FILE_TYPE_UNKNOWN; break;
        case 1: filetypeid = BV_TYPE_VMR; break;
        case 2: filetypeid = BV_TYPE_FMR; break;
        case 3: filetypeid = BV_TYPE_DMR; break;
        case 4: filetypeid = BV_TYPE_VTC; break;
        case 5: filetypeid = BV_TYPE_VMP; break;
  //      case 6: filetypeid = NIFTI_FILE_TYPE; break;
  //      case 7: filetypeid = NIFTI_PAIR_FILE_TYPE; break;
        default: filetypeid = FILE_TYPE_UNKNOWN;
    }
    return filetypeid;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 6. BATCH FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareBatchFuncs = function() {

    // set parameters for plugin
   BrainVoyager.SetPluginStringParameter("MenuOption", "BatchFuncs"); 
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MAIN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.initDlg = function() { 

    this.niftiDlg.windowTitle = "NIfTI converter"; 

    // ~~~ make nicknames for children ~~~
	var bv = BrainVoyager;
    startButton = bv.FindChild("btnStart"); 
    // global tab
    optConvertButton    = bv.FindChild("rbtnConvert");
    optInfoButton       = bv.FindChild("rbtnImgInfo");
    optPosButton        = bv.FindChild("rbtnPosFuncs");
    optManipButton      = bv.FindChild("rbtnHdrManip");
    optTrfButton        = bv.FindChild("rbtnTrfFuncs");
    optBatchButton      = bv.FindChild("rbtnBatchFunctions");
    getNiftiFileButton  = bv.FindChild("btnGetNIfTIFile");
    niftiNameLabel      = bv.FindChild("lblNIfTIname");
    
    // conversion tab
    niiFromButton = bv.FindChild("rbtnFromNII");  
    hdrFromButton = bv.FindChild("rbtnFromHDR");
    vmrFromButton = bv.FindChild("rbtnFromVMR"); 
    fmrFromButton = bv.FindChild("rbtnFromFMR"); 
    dmrFromButton = bv.FindChild("rbtnFromDMR"); 
    vtcFromButton = bv.FindChild("rbtnFromVTC"); 
    vmpFromButton = bv.FindChild("rbtnFromVMP");
    voiFromButton = bv.FindChild("rbtnFromVOI");
    niiToButton = bv.FindChild("rbtnToNII"); 
    hdrToButton = bv.FindChild("rbtnToHDR");
    vmrToButton = bv.FindChild("rbtnToVMR"); 
    fmrToButton = bv.FindChild("rbtnToFMR"); 
    dmrToButton = bv.FindChild("rbtnToDMR"); 
    vtcToButton = bv.FindChild("rbtnToVTC"); 
    vmpToButton = bv.FindChild("rbtnToVMP");
    transformBox = bv.FindChild("chbTrfImg");
    getBatchConvButton = bv.FindChild("btnBatchFiles");
    flipBox = bv.FindChild("chbFlip"); 
    batchBox = bv.FindChild("chbBatch"); 
    openinbvBox = bv.FindChild("chbOpenInbv"); 
    zeropadBox = bv.FindChild("chbZeropad");
    intensityBox = bv.FindChild("chbVMRintensity");
    vointerpBox = bv.FindChild("chboxVOIinterp");
    resetBox = bv.FindChild("chbReset");
    // information tab
    niiPrintButton = bv.FindChild("rbtnPrintNII"); 
    hdrPrintButton = bv.FindChild("rbtnPrintHDR");
    vmrPrintButton = bv.FindChild("rbtnPrintVMR"); 
    fmrPrintButton = bv.FindChild("rbtnPrintFMR"); 
    dmrPrintButton = bv.FindChild("rbtnPrintDMR"); 
    vtcPrintButton = bv.FindChild("rbtnPrintVTC"); 
    vmpPrintButton = bv.FindChild("rbtnPrintVMP");
    voiPrintButton = bv.FindChild("rbtnPrintVOI");
    destBothButton = bv.FindChild("rbtnDestBoth");
    destFileButton = bv.FindChild("rbtnDestFile"); 
    destLogButton = bv.FindChild("rbtnDestLog"); 
    // positioning tab
    getPosFileButton = bv.FindChild("btnGetPOSfile");
    createPosFileButton = bv.FindChild("rbtnCreatePosFile"); 
    posToImgButton = bv.FindChild("rbtnPosToImg"); 
    printPosMatButton = bv.FindChild("rbtnPrintMat"); 
    calcPosButton = bv.FindChild("rbtnComputePos");
    posFileLabel = bv.FindChild("lblPosFile");
    nxBox = bv.FindChild("nxBox");
    nyBox = bv.FindChild("nyBox");
    nzBox = bv.FindChild("nzBox");
    dxBox = bv.FindChild("dxBox");
    dyBox = bv.FindChild("dyBox");
    dzBox = bv.FindChild("dzBox");
    p00 = bv.FindChild("le00_p");
    p01 = bv.FindChild("le01_p");
    p02 = bv.FindChild("le02_p");
    p03 = bv.FindChild("le03_p");
    p10 = bv.FindChild("le10_p");
    p11 = bv.FindChild("le11_p");
    p12 = bv.FindChild("le12_p");
    p13 = bv.FindChild("le13_p");
    p20 = bv.FindChild("le20_p");
    p21 = bv.FindChild("le21_p");
    p22 = bv.FindChild("le22_p");
    p23 = bv.FindChild("le23_p");
    p30 = bv.FindChild("le30_p");
    p31 = bv.FindChild("le31_p");
    p32 = bv.FindChild("le32_p");
    p33 = bv.FindChild("le33_p");
    // header manipulation tab
    optCopyButton       = bv.FindChild("rbtnOptCopy");
    optFldManipButton   = bv.FindChild("rbtnOptManip");
    rbtnCopyPos         = bv.FindChild("rbtnCopyPos");
    rbtnCopyAll         = bv.FindChild("rbtnCopyAll");
    rbtnNiiType         = bv.FindChild("rbtnNiiType");
    rbtnHdrType         = bv.FindChild("rbtnHdrType");
    rbtnNiiTarget       = bv.FindChild("rbtnNiiTarget");
    rbtnHdrTarget       = bv.FindChild("rbtnHdrTarget");
    lblOtherNIfTIfile = bv.FindChild("lblOtherNIfTIfile");
    btnOtherNIfTIfile = bv.FindChild("btnOtherNIfTIfile");
    leQformCode = bv.FindChild("leQformCode");
    leQb = bv.FindChild("leQb");
    leQc = bv.FindChild("leQc");
    leQd = bv.FindChild("leQd");
    leQoffsetX = bv.FindChild("leQoffsetX"); 
    leQoffsetY = bv.FindChild("leQoffsetY");
    leQoffsetZ = bv.FindChild("leQoffsetZ");
    leSformCode = bv.FindChild("leSformCode");
    leSrow00 = bv.FindChild("leSrow00");
    leSrow01 = bv.FindChild("leSrow01");
    leSrow02 = bv.FindChild("leSrow02");
    leSrow03 = bv.FindChild("leSrow03");
    leSrow10 = bv.FindChild("leSrow10");
    leSrow11 = bv.FindChild("leSrow11");
    leSrow12 = bv.FindChild("leSrow12");
    leSrow13 = bv.FindChild("leSrow13");    
    leSrow20 = bv.FindChild("leSrow20");
    leSrow21 = bv.FindChild("leSrow21");
    leSrow22 = bv.FindChild("leSrow22");
    leSrow23 = bv.FindChild("leSrow23");
    leSrow30 = bv.FindChild("leSrow30"); // fixed 19-09-11
    leSrow31 = bv.FindChild("leSrow31"); // fixed 19-09-11
    leSrow32 = bv.FindChild("leSrow32"); // fixed 19-09-11
    leSrow33 = bv.FindChild("leSrow33"); // fixed 19-09-11
    // transformation tab
    invertTrfButton = bv.FindChild("rbtnInvertTrf");
    applyManTrfButton = bv.FindChild("rbtnApplyManTrf");
    applybvTrfButton = bv.FindChild("rbtnApplybvtrf");
    getTrfFileTypeBox = bv.FindChild("cmBoxTrfFiles");
    transformGrBox = bv.FindChild("grboxTrfMat");
    getTrfFileButton = bv.FindChild("btnGetTrfFile");
    trfFileLabel = bv.FindChild("lblTrfFile");
    t00 = bv.FindChild("le00"); 
    t01 = bv.FindChild("le01"); 
    t02 = bv.FindChild("le02"); 
    t03 = bv.FindChild("le03"); 
    t10 = bv.FindChild("le10"); 
    t11 = bv.FindChild("le11"); 
    t12 = bv.FindChild("le12"); 
    t13 = bv.FindChild("le13"); 
    t20 = bv.FindChild("le20"); 
    t21 = bv.FindChild("le21"); 
    t22 = bv.FindChild("le22"); 
    t23 = bv.FindChild("le23"); 
    t30 = bv.FindChild("le30"); 
    t31 = bv.FindChild("le31"); 
    t32 = bv.FindChild("le32"); 
    t33 = bv.FindChild("le33");     
    // batch tab
    writeBatchFileButton = bv.FindChild("rbtnWriteBatch");

    // connect widgets to functions here
    getBatchConvButton.clicked.connect(this, this.getBatchFile);    
    getNiftiFileButton.clicked.connect(this, this.getNiftiFile);
    btnOtherNIfTIfile.clicked.connect(this, this.getOtherNiftiFile);
    getPosFileButton.clicked.connect(this, this.getPosFile);    
    getTrfFileButton.clicked.connect(this, this.getTrfFile);
    startButton.clicked.connect(this, this.start);
   
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
