//
// Wrapper to Alard Roebroeck's GCMPlugin to run GCM's across subjects and to prepare map (names) for "Combine VMPs" or "ANCOVA"
// Rainer Goebel
// NOTE: Instead of "dlg.findChild()" we use "bv.FindChild()" since "findChild" currently not working with script extensions

var bv = BrainVoyager; // use short symbol to access BrainVoyager QX API

var scriptObj = new Object;

scriptObj.dialogFileName   = "RFXGCMPlugin.ui";  // specify name of file that will be used by BV to create your UI
scriptObj.dialogAccessName = "PluginDialog";         // specify the name you want to use for accessing your UI

scriptObj.initDlg = function()
{
	var dlg = this.PluginDialog; // less to write

	dlg.windowTitle = "RFX GCM"; // use to change dialog title
	
	// dlg.intensityRangeGroupBox.fromIntensitySpinBox.maximum = 255; // set max allowed value of "From" spin box

	// connec UI elements to script funcctions - you need to know the names of your elements (check your definition)
	dlg.quitButton.clicked.connect(this, this.onQuit);
	dlg.goButton.clicked.connect(this, this.onGo);
	bv.FindChild("loadVTCsFromMDMButton").clicked.connect(this, this.onLoadVTCsFromMDM);
	bv.FindChild("selectVOIFileBtn").clicked.connect(this, this.onSelectVOIFile);
	bv.FindChild("addVTCBtn").clicked.connect(this, this.onAddVTCFile);
	bv.FindChild("clearVTCsBtn").clicked.connect(this, this.onClearVTCs);
	bv.FindChild("SingleIntervalPerVTCGrpBx").clicked.connect(this, this.onClickedIntervalMethod);
	bv.FindChild("ProtocolConditionPerVTCGrpBx").clicked.connect(this, this.onClickedProtocolMethod);
	bv.FindChild("ConditionsLsWdg").itemSelectionChanged.connect(this, this.onConditionChanged);
	bv.FindChild("subjectSpecificVOIsCheckBox").clicked.connect(this, this.onSwitchVOIOption);
	
	bv.SetPluginStringParameter("COMMAND", "StartCheck");
	var VOIFile = bv.VOIFileName;
	bv.SetPluginStringParameter("VOIFile", VOIFile); // if there is a VOI file, load and show VOI names in ListWidget
	if( !bv.ExecutePlugin() ) // prereqs not met
		this.onQuit();
		
	this.fillVOIList();
}

scriptObj.onQuit = function()
{
	this.PluginDialog.close();
}

scriptObj.onGo = function()
{
	var dlg = this.PluginDialog; // less to write
	
	var VOIsLW = bv.FindChild("VOIsLsWdg");
	var VTCsLW = bv.FindChild("VTCsLsWdg");
	var SubjVOIsOp = bv.FindChild("subjectSpecificVOIsCheckBox");
		
	var n_vois = VOIsLW.count;
	if(n_vois == 0)
	{
		bv.MessageBox("No VOIs available!");
		return;
	}

	// get selected VOI:
	var sel_voi = VOIsLW.currentRow; // subject-specific VOIs now supported, see below
	if(sel_voi < 0)
	{
		bv.MessageBox("No VOI selected - please select the VOI serving as reference region for GCM calculation!")
		return;
	}
	bv.SetPluginIntParameter("SelectedVOIIndex", sel_voi); // tell plugin which VOI has been selected - overwritten below if subject-specific VOI

	var n_vtcs = VTCsLW.count;
	if(n_vtcs == 0)
	{
		bv.MessageBox("No VTC files available!")
		return;
	}
	
	if(bv.FindChild("SingleIntervalPerVTCGrpBx").checked)
		bv.SetPluginIntParameter("SelectedTimeWindowMethod", 1); // interval
	else
		bv.SetPluginIntParameter("SelectedTimeWindowMethod", 2); // protocol condition
	
	bv.SetPluginStringParameter("SelectedConditionName", bv.FindChild("ConditionNameEd").text);

	var docVMR = bv.ActiveDocument;
	
	// analyze initial substring to find out how many VTCs per subject, then run GCM once for each subject with one or more VTCs
	var SubjIDs = new Array();
	var n_subjects = 0;
	for(var i=0; i<n_vtcs; i++)
	{
		var VTCPathFile = VTCsLW.item(i).text();
		var VTCFile = this.extractFilenameFromPath(VTCPathFile);
		var s = this.getSubjectID(VTCFile);
		if(s == "") return; // error msg
		if(this.arrayContains(SubjIDs, n_subjects, s) == false) // if subj-id not in array, add it
		{
			//bv.PrintToLog(s);
			SubjIDs[n_subjects] = s;
			n_subjects++;
		}
	}
	bv.PrintToLog("No. of subjects: " + n_subjects);
	bv.SetPluginIntParameter("NrOfSubjects", n_subjects);
	
	if(bv.FindChild("SingleIntervalPerVTCGrpBx").checked)
	{
		bv.SetPluginIntParameter("VTCIntervalStart", bv.FindChild("IntervalFromSpBx").value);
		bv.SetPluginIntParameter("VTCIntervalEnd",   bv.FindChild("IntervalToRelToNSpBx").value);
	}
	else
	{
		if(this.checkProtocolsOfAllVTCs() == false)
			return;

		bv.SetPluginIntParameter("VTCProtocolCond", bv.FindChild("ConditionsLsWdg").currentRow);
	}
	
	// in case that subject-specific VOIs selected, check that for each subject there is a VOI defined
	// put all original VOIs in an array to more easier process VOIs below
	var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");
	var OrigVOIs = new Array();
	var n_orig_vois = 0;
	for(var i=0; i<NVOIs; i++)
	{
		var VOINameID = "NameOfVOI-" + (i+1);
		var VOIName = bv.GetPluginStringParameter(VOINameID);
		OrigVOIs[n_orig_vois] = VOIName;
		n_orig_vois++;
	}

	if(SubjVOIsOp.checked)
	{
		// var SelectedPureVOI = bv.GetPluginStringParameter("SelectedPureVOIName"); // only correct when updated in case user selects a pure VOI in VOIsLW
		var pure_voi_index = VOIsLW.currentRow;
		if(pure_voi_index < 0) return;
		var SelectedPureVOI = VOIsLW.item(pure_voi_index).text();
		
		for(var s=0; s<n_subjects; s++)
		{
			var sName = SubjIDs[s];
			
			var constructed_voi = sName + "_" + SelectedPureVOI;
			
			if(this.arrayContains(OrigVOIs, n_orig_vois, constructed_voi) == false) // if expected VOI does not exist, stop
			{
				bv.MessageBox("Can not proceed - expected VOI - " + constructed_voi + " - not found!");
				return;
			}
		}
	}
	
	for(var i=0; i<n_subjects; i++)
	{
		var s = SubjIDs[i];
		
		if(SubjVOIsOp.checked)
		{
			var constructed_voi = s + "_" + SelectedPureVOI;
			for(var voi_i=0; voi_i<n_orig_vois; voi_i++)
			{
				if(OrigVOIs[voi_i] == constructed_voi)
				{
					bv.SetPluginIntParameter("SelectedVOIIndex", voi_i); // tell plugin which VOI has been selected
					
					// associate subject with POI name to simplify access of subject POI name in plugin code
					var SubjVOIIndex = "SubjVOIName-" + (i+1);
					bv.SetPluginStringParameter(SubjVOIIndex, constructed_voi);
					
					break;
				}
			} 
		}
		
		var n_subj_vtcs = 0;
		
		for(var vtc=0; vtc<n_vtcs; vtc++)
		{
			var VTCPathFile = VTCsLW.item(vtc).text();
			var VTCFile = this.extractFilenameFromPath(VTCPathFile);
		
			if(this.getSubjectID(VTCFile) == s) // VTC belongs to current subject
			{
				n_subj_vtcs++;
				var SubjVTCNameID = "SubjVTC-" + n_subj_vtcs;
				bv.SetPluginStringParameter(SubjVTCNameID, VTCPathFile);

				if(n_subj_vtcs == 1) // if((i==0) && (vtc==0)) // changed: link always correct one to set default folder to appropriate subject folder
				{
					docVMR.LinkVTC(VTCPathFile); // since GCM plugin needs (any) VTC, link first one
					
					// new: we also save for each subject folder (of first VTC) for plugin to allow case that data is not in one folder
					var SubjFirstVTCFullFileID = "SubjDataFolder-" + (i+1);
					bv.SetPluginStringParameter(SubjFirstVTCFullFileID, VTCPathFile);
				}
			}
		}
		bv.PrintToLog(s + " has " + n_subj_vtcs + " subj vtcs"); 
		
		bv.SetPluginIntParameter("NrOfSubjVTCs", n_subj_vtcs);
		
		// run once for each subject
		bv.SetPluginStringParameter("SubjectID", s);
		bv.SetPluginStringParameter("COMMAND", "RunGCM");
		bv.ExecutePlugin();
	}

	// finally, make a single NrVMP containing all subj maps (one for dGCM, one for Inst)
	for(var i=0; i<n_subjects; i++)
	{
		var s = SubjIDs[i];
		var SubjNameID = "SubjID-" + (i+1);
		bv.SetPluginStringParameter(SubjNameID, s);		
	}
	if(SubjVOIsOp.checked)
	{ 
		bv.SetPluginIntParameter("SubjectSpecificVOIs", 1); 
	}
	else 
	{ 
		bv.SetPluginIntParameter("SubjectSpecificVOIs", 0); 
	}

	bv.SetPluginStringParameter("COMMAND", "PrepareForANCOVA");
	bv.ExecutePlugin();
	
	docVMR.ShowVolumeMapsDlg();
	
	this.PluginDialog.close();
}

scriptObj.onSelectVOIFile = function()
{
	var VOIFileName = bv.BrowseFile("Select VOI File", "VOI files (*.voi)");
		
	var dlg = this.PluginDialog; // less to write
		
	if(VOIFileName == "")
	{
		bv.FindChild("VOIFileEd").text = "";
		bv.SetPluginStringParameter("VOIFile", "");
		bv.FindChild("VOIsLsWdg").clear();
		return;
	}
		
	// let plugin load VOI file:
	bv.SetPluginStringParameter("VOIFile", VOIFileName);
	bv.SetPluginStringParameter("COMMAND", "StartCheck"); // use special command, e.g "LoadVOIFile"
	bv.ExecutePlugin();
	
	this.fillVOIList();
}

scriptObj.fillVOIList = function()
{	
	var dlg = this.PluginDialog; // less to write
	var VOIsLW = bv.FindChild("VOIsLsWdg");
	var VOIEd  = bv.FindChild("VOIFileEd");

	var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");

	VOIEd.text = bv.VOIFileName;
	VOIsLW.clear();

	for(var i=0; i<NVOIs; i++)
	{
		var VOINameID = "NameOfVOI-" + (i+1);
		var VOIName = bv.GetPluginStringParameter(VOINameID);
		VOIsLW.addItem(VOIName);
	}
	if(NVOIs > 0)
		VOIsLW.currentRow = 0;
}

scriptObj.onAddVTCFile = function()
{
	var VTCFileName = bv.BrowseFile("Select VTC File", "VTC files (*.vtc)");
	
	bv.PrintToLog("Script>  Selected VTC File: " + VTCFileName);
	
	bv.FindChild("VTCsLsWdg").addItem(VTCFileName);
}

scriptObj.onLoadVTCsFromMDM = function()
{
	var MDMFileName = bv.BrowseFile("Select MDM File", "MDM files (*.mdm)");
	
	var VTCList = new Array();
	VTCList = bv.GetVTCsOfMDM(MDMFileName);

	var VTCsLW = bv.FindChild("VTCsLsWdg");
	VTCsLW.clear();
	VTCsLW.addItems(VTCList);
}

scriptObj.onClearVTCs = function()
{
	bv.FindChild("VTCsLsWdg").clear();
}
	
// helper functions
//
scriptObj.getSubjectID = function(str)
{
	var idx = str.indexOf("_");
	if(idx >= 0)
		return str.substr(0, idx);
	else
		return "";
}

scriptObj.getPureVOI = function(str)
{
	var idx = str.indexOf("_");
	if(idx >= 0)
		return str.substr(idx+1, str.length);
	else
		return "";
}

scriptObj.arrayContains = function(arr, n, el)
{
	for(var i=0; i<n; i++)
	{
		if(arr[i] == el)
			return true;
	}
	return false;
}

scriptObj.extractFilenameFromPath = function(pathfile)
{
	var idx = pathfile.lastIndexOf("/");
	if(idx >= 0)
		return pathfile.substr(idx+1);
	else
		return pathfile
}

scriptObj.onClickedIntervalMethod = function()
{
	bv.FindChild("ProtocolConditionPerVTCGrpBx").setChecked(false);
	
	bv.FindChild("ConditionNameEd").text = "interval";
}

scriptObj.onClickedProtocolMethod = function()
{
	var VTCsLW  = bv.FindChild("VTCsLsWdg");
	var CondsLW = bv.FindChild("ConditionsLsWdg");
	CondsLW.clear();
	
	var n_vtcs = VTCsLW.count;
	if(n_vtcs == 0)
	{
		bv.MessageBox("No VTC files available! Add VTC files before switching to this time interval specification method");
		bv.FindChild("ProtocolConditionPerVTCGrpBx").setChecked(false);
		return;
	}
	
	// show all conditions of protocol of first VTC for selection (when running RFX-GCM, check that all subsequent protocols have same conds as first VTC, see below)
	var VTCPathFile = VTCsLW.item(0).text();
	// let plugin provide conditions of VTC's protocol 
	bv.SetPluginStringParameter("VTCFile", VTCPathFile);
	bv.SetPluginStringParameter("COMMAND", "GetCondsOfVTCProtocol");
	var ret = bv.ExecutePlugin();
	if(!ret)
	{
		bv.MessageBox("Error getting conditions from VTCs protocol - protocol attached to all VTCs?");
		return;
	}
	
	var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
	for(var c=0; c<NrOfConditions; c++)
	{
		var CondNameID = "CondName-" + (c+1);
		var CondName = bv.GetPluginStringParameter(CondNameID);
		CondsLW.addItem(CondName);
	}
	if(NrOfConditions > 1)
		CondsLW.currentRow = 1;
	else if(NrOfConditions > 0)
		CondsLW.currentRow = 0;

	bv.FindChild("SingleIntervalPerVTCGrpBx").setChecked(false);
}

scriptObj.onConditionChanged = function()
{
	var CondsLW = bv.FindChild("ConditionsLsWdg");

	var idx = CondsLW.currentRow;
	if(idx >= 0)
		bv.FindChild("ConditionNameEd").text = CondsLW.item(idx).text();
}

scriptObj.onSwitchVOIOption = function()
{
	var VOIsLW = bv.FindChild("VOIsLsWdg");
	var SubjVOIsOp = bv.FindChild("subjectSpecificVOIsCheckBox");
	
	if(SubjVOIsOp.checked)
	{
		VOIsLW.clear();

		// split subject ID and pure VOI name assuming "<SubjID_VOI>" naming scheme
		var NVOIs = bv.GetPluginIntParameter("NrOfVOIsAvailable");
		var SubjIDs = new Array();
		var PureVOIs = new Array();
		var n_subjects = 0;
		var n_pure_vois = 0;
		
		for(var i=0; i<NVOIs; i++)
		{
			var VOINameID = "NameOfVOI-" + (i+1);
			var VOIName = bv.GetPluginStringParameter(VOINameID);

			var s = this.getSubjectID(VOIName); // substring before "_"
			if(s == "") return; // error msg
			var v = this.getPureVOI(VOIName);   // substring after "_"

			if(this.arrayContains(SubjIDs, n_subjects, s) == false) // if subj-id not in array, add it
			{
				//bv.PrintToLog(s);
				SubjIDs[n_subjects] = s;
				n_subjects++;
			}
			
			if(this.arrayContains(PureVOIs, n_pure_vois, v) == false) // if pure voi not in array, add it
			{
				//bv.PrintToLog(v);
				PureVOIs[n_pure_vois] = v;
				n_pure_vois++;
			}
		}

		VOIsLW.addItems(PureVOIs);
		
		if(n_pure_vois > 0)
		{
			VOIsLW.currentRow = 0;
			
			bv.SetPluginStringParameter("SelectedPureVOIName", PureVOIs[0]); // not used, can be deleted; was: from subj-ID's and pure VOI name, we can get original VOI
		}
		else
		{
			bv.SetPluginStringParameter("SelectedPureVOIName", "");
			
			// no pure VOIs -> turn off
			SubVOIsOp.checked = false;
			this.fillVOIList;
		}
	}
	else
	{
		this.fillVOIList();
	}
}

scriptObj.checkProtocolsOfAllVTCs = function()
{
	var VTCsLW  = bv.FindChild("VTCsLsWdg");
	var CondsLW = bv.FindChild("ConditionsLsWdg");
	
	var n_vtcs = VTCsLW.count;
	if(n_vtcs == 0)
	{
		bv.MessageBox("No VTC files available!");
		return false;
	}
	var n_conds = CondsLW.count;
	if(n_conds == 0)
	{
		bv.MessageBox("No protocol conditions available!");
		return false;
	}
	var sel_cond_idx  = CondsLW.currentRow;
	var sel_cond_name = CondsLW.currentItem().text();
	
	// check that all VTCs have a protocol with same condition as selected one for first VTC
	for(var i=0; i<n_vtcs; i++)
	{
		var VTCPathFile = VTCsLW.item(i).text();
		// let plugin provide conditions of VTC's protocol 
		bv.SetPluginStringParameter("VTCFile", VTCPathFile);
		bv.SetPluginStringParameter("COMMAND", "GetCondsOfVTCProtocol");
		var ret = bv.ExecutePlugin();
		if(!ret)
		{
			bv.MessageBox("Error getting conditions from VTCs protocol - protocol attached to all VTCs?");
			return false;
		}
		
		var NrOfConditions = bv.GetPluginIntParameter("NrOfConditions");
		if(NrOfConditions < n_conds)
			return false;
		var CondNameID = "CondName-" + (sel_cond_idx+1);
		var CondName = bv.GetPluginStringParameter(CondNameID);
		if(CondName != sel_cond_name)
		{
			bv.MessageBox("Protocol of VTC " + VTCPathFile + " does not have the selected condition, can not determine condition intervals.");
			return false;
		}		
	}

	return true;
}
	
// It is important to return script object containing properties and functions to be called from BrainVoyager QX
// 
returnScriptObj = function()
{
	return scriptObj;
}
returnScriptObj(); // we call this at the end to return created script object to BrainVoyager QX
