import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt

doc = BrainVoyager.ActiveDocument              # A FMR must be the current document 
dimX = doc.DimX
dimY = doc.DimY
dimZ = doc.DimZ
dimT = doc.NrOfVolumes

fmrData = doc.GetFMRDataAsDoubleList()         # returns tuple (not list) at present

data   = np.array(fmrData)                     # convert list (or tuple) to numpy array
data4D = data.reshape(dimT, dimZ, dimY, dimX)  # reshape 1D to 4D array

def calculate_tSNR(array4D):
    m    = array4D.mean(axis=0)
    sd   = array4D.std(axis=0)
    tsnr = m / sd
    return m, sd, tsnr

# plot a slice with original, mean, sdev and TSNR image
def plotslice(sl):
    plt.subplot(221)
    plt.imshow(data4D[0][sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.title('Original image (first volume)')
    plt.subplot(222)
    plt.imshow(M[sl])
    plt.colorbar()
    plt.title('Mean time course image')
    plt.subplot(223)
    plt.imshow(SD[sl])
    plt.colorbar()
    plt.title('StdDev time course image')
    plt.subplot(224)
    plt.imshow(TSNR[sl])
    plt.colorbar()
    plt.title('TSNR image')
    plt.show()

M, SD, TSNR = calculate_tSNR(data4D)
plotslice(13)