import numpy as np
import scipy.ndimage as ndimage
import matplotlib.pyplot as plt
import bv

doc = BrainVoyager.ActiveDocument               # A VMR should be the current document 
n_voxels = doc.DimX * doc.DimY * doc.DimZ

# "vmrarray" function of _bv module provides direct access to VMR volume as ndarray
# this solves the very slow voxelwise access via scripting (see PythonScripts/VolumeVMR_SlowAccess.py)
#
arr = bv.vmrarray(doc)

arrf = arr.astype(np.float32)       # convert to float32 array
arr2 = arrf.reshape(doc.DimX, doc.DimY, doc.DimZ) # reshape 1D to 3D array
arr3 = ndimage.gaussian_filter(arr2, sigma=3)     # smooth volume
arr3_ui8 = arr3.astype(np.uint8)                  # convert smoothed arrat back to uint8 array
bv.set_vmrarray(doc, arr3_ui8)                    # copy array to BV internal volume (C array)
doc.UpdateWindow(True)                            # update display to see inverted intensities in BV doc view

def plotslice(sl):  # plot result for a given slice
    plt.subplot(221)
    plt.imshow(arr2[sl])
    plt.colorbar()
    plt.subplot(222)
    plt.imshow(arr2[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.subplot(223)
    plt.imshow(arr3[sl])
    plt.colorbar()
    plt.subplot(224)
    plt.imshow(arr3[sl], cmap=plt.cm.gray)
    plt.colorbar()
    plt.show()

plotslice(140)
