"""
Demonstration of new surface map script functions introduced in BV 20.6

A VMR must be available with a overlaid volume map (VMP) using GUI or script commands

"""
doc = BrainVoyager.ActiveDocument
mesh = doc.CurrentMesh

mesh.CreateSurfaceMapFromVolumeMap(2, True)
# this version samples VMP at locations of mesh vertices
# param 1: interpolation method, 1 -> nearest nbr, 2 -> trilinear
# param 2: sample only non-zero values (True recommended)

mesh.CreateSurfaceMapFromVolumeMapDepth(2, True, False, -1.0, 3.0, 1.0)
# this version samples in depth around mesh vertices along vertex normal
# param 1: interpolation method, 1 -> nearest nbr, 2 -> trilinear
# param 2: sample only non-zero values (True recommended)
# param 3: True -> return max value encountered, False -> return average of encountered values
# param 4: Start depth value relative to mesh vertex along normal in WM-GM direction [mm]
# param 5: End depth value relative to mesh vertex along normal in WM-GM direction [mm]
# param 6: Step size for sampling from depth start to depth end value along normal [mm]
# parameters 4-6 will be converted into voxel resolution using VMR's resolution (assuming iso-voxel data)

# call "UpdateAppearance()" or "ShowSurfaceMap(smp_idx)" to overlay created surface map and to update opengl window
mesh.UpdateAppearance()

# example calls
mesh.CreateSurfaceMapFromVolumeMapDepth(2, True, True, -1.0, 3.0, 1.0)
mesh.CreateSurfaceMapFromVolumeMapDepth(1, True, True,  0.0, 2.5, 0.5)
mesh.UpdateAppearance()

# mesh smoothing
#mesh.SmoothCurrentMap(100) 
# this (old) version uses current map, which is the last overlaid map, if available
# param 1: number of smooth cycles using values of nearest neighbors 

map_idx = mesh.NrOfSurfaceMaps - 1  # select last available map
mesh.SmoothMap(map_idx, 100, True, True, False)
# this new version allows to specify map index, e.g. to enumerate and smooth all maps
# this version also allows to restrict vertices (neighbors) used for smoothing
# param 1: index of map to be smoothed (in range: 0 to mesh.NrOfSurfaceMaps - 1)
# param 2: number of smooth cycles using values of nearest neighbors 
# param 3: True -> restrict smoothing to vertices with values unequal zero (zero considered as "missing value")
# param 4: True -> restrict inclusion of neighbors to those with values unequal zero
# param 5: True -> restrict smoothing to vertices with values > map threshold (generalized version of param 3)

mesh.UpdateAppearance()

# note that circular angle maps are smoothed circular automatically
# for smoothing lags of coross-corr maps, use this command:
# mesh.SmoothMapLags(map_idx, 100, True, True, True, False) 
# param 1: index of map to be smoothed (in range: 0 to mesh.NrOfSurfaceMaps - 1)
# param 2: number of smooth cycles using values of nearest neighbors 
# param 3: True -> smooth lags circular, False -> smooth by simple averaging
# param 4: True -> restrict smoothing to vertices with values unequal zero (zero considered as "missing value")
# param 5: True -> restrict inclusion of neighbors to those with values unequal zero
# param 6: True -> restrict smoothing to vertices with values > map threshold (generalized version of param 3)
