import PythonQt.Qt as qt

import os
import numpy as np
import scipy as sp
import scipy.misc
import scipy.integrate

from matplotlib.figure import Figure
from matplotlib.backends.backend_agg import FigureCanvasAgg

fig = Figure(figsize=(6, 6))
ax = fig.add_subplot(111)

def f(x):
    return x**3 - 10*np.sin(x) - 4

def df(x):
    return sp.misc.derivative(f, x)

@np.vectorize
def F(x):
    return sp.integrate.quad(f, 0, x)[0]

def make_fig():
    X = np.linspace(-3, 3, 100)
    Y  =  f(X)
    Y1 = df(X)
    Y2 =  F(X)
    ax.plot(X,  Y, linewidth=2, label="f")
    ax.plot(X, Y1, linewidth=2, linestyle='dashed', label="f'")
    ax.plot(X, Y2, linewidth=2, linestyle='dotted', label="F")
    ax.legend()

def create_fig_dlg():
    canvas = FigureCanvasAgg(fig)
    canvas.print_figure(os.path.join(os.getcwd(), '.matplotlib-for-qt.png')) # current solution: save matplotlib-fig to disk and reload as QImage below
    img = qt.QImage()
    ok = img.load(os.path.join(os.getcwd(), '.matplotlib-for-qt.png'))
    if ok == False:
        return
    dlg = qt.QWidget()
    dlg.setAttribute(55, True) # 55 -> WA_DeleteOnClose
    dlg.setWindowTitle("matplotlib in Qt GUI")
    box = qt.QVBoxLayout(dlg)
    label = qt.QLabel(dlg)
    label.setContentsMargins(0, 0, 0, 0)
    box.setContentsMargins(0, 0, 0, 0)
    box.addWidget(label)
    pxm = qt.QPixmap()
    pxm.convertFromImage(img)
    label.setPixmap(pxm)
    label.setScaledContents(True)
    return dlg

make_fig()
dlg1 = create_fig_dlg()
dlg1.show()

#canvas = FigureCanvasAgg(fig)
#canvas.print_figure("test.png", dpi=200) 