#
# This script shows how a mesh can be smoothed and inflated.
# It is assumed that a "*_RECO.srf" mesh is availalbe (e.g. loaded)
#
#  author: Rainer Goebel
#

def MorphMesh():
    docVMR = BrainVoyager.ActiveDocument
    if docVMR == None:
        return
    mesh = docVMR.CurrentMesh
    if mesh == None:
        return
    mesh.MorphingUpdateInterval = 25
    # mesh.SmoothMesh(30, 0.07);   # standard smoothing - mesh will shrink
    mesh.SmoothRecoMesh(50, 0.07)  # special "high-frequency" smoothing removing jags of reconstructed voxel borders without shrinking mesh 
    mesh.CalculateCurvature()
    mesh.SmoothCurrentMap(5)
    mesh.MeshScene.UpdateSurfaceWindow()
    new_name = remove_extension(mesh.FileName) + 'SM.srf'
    mesh.SaveAs(new_name)
    mesh.InflateMesh(500, 0.8, ''); # if '' used for 3rd param (area reference mesh), the current mesh at the moment the fn is called is used to calculate ref mesh area

# helpers
def remove_extension(filename):
    ext_length = get_extension_length(filename)
    name = ''
    for i in range(len(filename) - ext_length):
        letter = filename[i]
        name += letter
    return name

def get_extension_length(filename):
    i = len(filename) - 1
    while i > 0:
        letter = filename[i]
        if letter == ".":
            dot_pos = i			
            break
        i -= 1
    extension_length = len(filename) - dot_pos
    return extension_length

MorphMesh()