
# Example of basic preprocessing steps for a single VMR document
# More realistic preprocessing would loop over subjects and sessions.
# For information about the parameters of the commands, see topic "VMR Processing Commands"
# in the "Scripting" chapter of the "Python Developer Guide"

# we simply use current doc, which should be a non-preprocessed VMR document
# usually the document is created or opened via script command

docVMR = BrainVoyager.ActiveDocument

# The VMR data can be iso-voxeled, (only executes if needed)
pathFileName = docVMR.PathFileName
newPathFileName = pathFileName[0:-4] + "_ISO.vmr"
print newPathFileName
docVMR.AutoTransformToIsoVoxel(2, newPathFileName)

# The VMR data can be oriented into BV's standard sagittal space (if needed)
pathFileName = docVMR.PathFileName
newPathFileName = pathFileName[0:-4] + "_SAG.vmr"
print newPathFileName
docVMR.AutoTransformToSAG(newPathFileName)

# correct intensity inhomogeneities including brain extraction
docVMR.CorrectIntensityInhomogeneities()

MNISpace = True
TALSpace = False

if MNISpace:
    # transform VMR from native space to MNI152 space
    docVMR.NormalizeToMNISpace()

if TALSpace:
    # automatically convert VMR from native space to ACPC and Talairach space
    docVMR.AutoACPCAndTALTransformation()

# documents can be identified by name and selectively closed if desired
n_docs = BrainVoyager.Documents.Size()
for idx in range(0, n_docs):
    doc = BrainVoyager.Documents.Item(idx)
    print(doc.FileName)

# TAL normalization: close the original (inhomogeneity corrected) VMR
if TALSpace:
    docVMR.Close()

# get MNI or TAL document object to further work in normalized space
docNormalized = BrainVoyager.ActiveDocument
print(docNormalized.PathFileName)
