import numpy as np
#import matplotlib as mpl
#mpl.use('macosx')    # use() must be called before pyplot can be imported
#print(mpl.get_backend())
import matplotlib.pyplot as plt

doc = BrainVoyager.ActiveDocument   # A VMR should be the current document 

n_voxels = doc.DimX * doc.DimY * doc.DimZ

# Run this only if you have a couple of minutes... inverts the intensities of VMR
# The use of voxel-wise "GetVoxelIntensity" BV funtion in large nested loops is VERY slow!
# For plugin development, fast access to arrays is directly provided (see e.g. VMR_FastArrayAccess.py)

volume = np.empty(n_voxels, dtype=np.float32)
vol3D  = volume.reshape(doc.DimX, doc.DimZ, doc.DimY)

for x in range(doc.DimX):
    for y in range(doc.DimY):
        for z in range(doc.DimZ):
            vol3D[x, y, z] = doc.GetVoxelIntensity(x, y, z)

vol3D = 225.0 - vol3D  # vectorized math is fast!

for x in range(doc.DimX):
    for y in range(doc.DimY):
        for z in range(doc.DimZ):
            doc.SetVoxelIntensity(x, y, z, vol3D[x, y, z])

doc.UpdateWindow()            # update doc view to show result

# show slices using matplotlib
vol3D = 225.0 - vol3D             # undo intensity flip
vol3Dsag = np.swapaxes(vol3D,1,2) # to get upright SAG view
plt.subplot(223)
plt.imshow(vol3Dsag[doc.DimX//2], cmap=plt.cm.gray)
plt.subplot(221)
plt.imshow(vol3Dsag[doc.DimX//2])
vol3Dcor = np.swapaxes(vol3D,0,2) # to get upright COR view
plt.subplot(222)
plt.imshow(vol3Dcor[:,doc.DimY//2,:])
vol3Dtra = np.swapaxes(vol3D,0,1) # to get upright TRA view
plt.subplot(224)
plt.imshow(vol3Dtra[:,:,doc.DimZ//2])
plt.show()
