
/* 	CreateVTCfiles.qs
	Script to create VTC files 
	By Rainer Goebel 2005
	Modified for QX 2.1 by Hester Breman 2009
	Modified for QX 2.4 by Rainer Goebel 2012
*/

/* This information is used in the functions */
	var ObjectsRawDataPath = BrainVoyager.PathToSampleData + "ObjectsDicomGSG/";
	var DTIDataPath = BrainVoyager.PathToSampleData + "Human31dir/";
	var today = new Date();
	var nameFMR = ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_SCCAI_3DMCTS_SD3DSS4.00mm_THPGLMF2c.fmr";
	var nameVMRinNative = ObjectsRawDataPath +"CG_3DT1MPR_SCRIPT_IIHC.vmr";
	var nameVMRinACPC = ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_IIHC_aACPC.vmr";
	var nameVMRinTAL = ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_IIHC_aTAL.vmr";
	var nameIAfile = ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_SCCAI_3DMCTS_SD3DSS4.00mm_THPGLMF2c-TO-CG_3DT1MPR_SCRIPT_IIHC_IA.trf";
	var nameFAfile = ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_SCCAI_3DMCTS_SD3DSS4.00mm_THPGLMF2c-TO-CG_3DT1MPR_SCRIPT_IIHC_FA.trf";
	var nameACPCfile = ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_IIHC_aACPC.trf";
	var nameTALfile = ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_IIHC_aACPC.tal";
	var nameVTCinNative= ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_NATIVE.vtc";
	var nameVTCinACPC = ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_ACPC.vtc";
	var nameVTCinTAL= ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_TAL.vtc";
	var dataType = 2; // 1: int16, 2: float32
	var resolution = 2; // one of 1, 2 or 3 mm^2
	var interpolation = 1; 
	var threshold = 100; 
	var extendedBoundingBox = false;

/* 	This code is executed when clicking 'Run' 
	If an error occurs, it is printed to the BrainVoyager QX Log tab 
*/
	BrainVoyager.ShowLogTab();
	BrainVoyager.PrintToLog("Start creating VTCs...");
	try {
		//CreateVMRinNativeSpace(); 
		//CreateVMRinAcpcSpace();
		CreateVMRinTalairachSpace(extendedBoundingBox); // make a VTC in Talairach space with extended bounding box = false
		//extendedBoundingBox = true;
		//var nameVTCinTAL= ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_TAL_ext.vtc"; // rename, so that the file won't be overwritten
		//CreateVMRinTalairachSpace(extendedBoundingBox); // make a VTC in Talairach space with extended bounding box (in z-dir)
	} catch (e) {
		BrainVoyager.PrintToLog("Error: " + e);
	}

/* These functions are invoked from the section above */
	function CreateVMRinNativeSpace() 
	{
		var docVMR = BrainVoyager.OpenDocument(nameVMRinNative);
		docVMR.ExtendedTALSpaceForVTCCreation = false; // this is true or false
		var success = docVMR.CreateVTCInVMRSpace(nameFMR, nameIAfile, nameFAfile, nameVTCinNative, dataType, resolution, interpolation, threshold);
		docVMR.Close();
	}

	function CreateVMRinAcpcSpace() 
	{
		var docVMR = BrainVoyager.OpenDocument(nameVMRinNative);
		docVMR.ExtendedTALSpaceForVTCCreation = false; // this is true or false
		var success = docVMR.CreateVTCInACPCSpace(nameFMR, nameIAfile, nameFAfile, nameACPCfile, nameVTCinACPC, dataType, resolution, interpolation, threshold);
		docVMR.Close();
	}

	function CreateVMRinTalairachSpace(useExtendedBoundingBox) 
	{
		var docVMR = BrainVoyager.OpenDocument(nameVMRinNative);
		docVMR.ExtendedTALSpaceForVTCCreation = useExtendedBoundingBox; // this is true or false

		// new in v2.4.1: specify bounding box for target VTC (works for any target reference space)
		docVMR.UseBoundingBoxForVTCCreation = true; // use bounding box
		// use properties to read and set bounding box values (here we create VTC only in lower posterior part of brain):
		docVMR.TargetVTCBoundingBoxZStart = 110; // values will be adjusted to fit on multiple of resolution
		docVMR.TargetVTCBoundingBoxZEnd  = 150; // values not changed (here X/Y) use default (TAL) bounding box values
		docVMR.TargetVTCBoundingBoxYStart = 128;

		var success = docVMR.CreateVTCInTALSpace(nameFMR, nameIAfile, nameFAfile, nameACPCfile, nameTALfile, nameVTCinTAL, dataType, resolution, interpolation, threshold);
		
		docVMR.Close();

		var docVMRTAL = BrainVoyager.OpenDocument(nameVMRinTAL);
		docVMRTAL.LinkVTC(nameVTCinTAL);
	}
