//
// This script shows how to prepare and run (rigid) CBA including group mesh/curvature map visualization.
//
//  Created by Rainer Goebel, May 2013
//

function CreateSphereFromFoldedMesh()
{
	var ok;
	var docVMR = BrainVoyager.ActiveDocument; if(docVMR == undefined) return;
	// It is assumed that a folded "*_RECOSM.srf" mesh is availalbe (e.g. loaded)
	var mesh = docVMR.CurrentMesh; if(mesh == undefined) return;

	mesh.MorphingUpdateInterval = 50;
	mesh.CalculateCurvatureCBA();
	mesh.SmoothCurrentMap(5);
	mesh.MeshScene.UpdateSurfaceWindow();
	mesh.InflateMeshToSphere(800);
	mesh.CorrectInflatedSphereMesh(3000);
	var new_name = removeLastUnderscoreSegment(mesh.FileName) + "_SPHERE.srf";
	mesh.SaveAs(new_name);
}

function MapSphereGetCurvature()
{
	var ok;
	var docVMR = BrainVoyager.ActiveDocument; if(docVMR == undefined) return;
	var meshScene = docVMR.MeshScene; if(meshScene == undefined) return;
	var mesh = meshScene.CurrentMesh; if(mesh == undefined) return;

	// we assume created sphere mesh (see "CreateSphereFromFoldedMesh" above) is available as current mesh (created or loaded) AND available on disk (since it is reloaded)
	var sphere_mesh_name =  mesh.FileName;
	// we need name of folded original mesh below - here we assume that name is same as SPHERE mesh but instead of "SPHERE" has "RECOSM" at end of name
	var folded_mesh_name = removeLastUnderscoreSegment(sphere_mesh_name) + "_RECOSM.srf";

	meshScene.SphereResolutionCBA = 1; // 1 -> standard resolution (default), 2 -> low resolution, 3 -> high resolution
	var saved_ssm_file          = meshScene.MapSphereMeshFromStandardSphere();
	var saved_foldedSPH_file = meshScene.SetStandardSphereToFoldedMesh(folded_mesh_name);
	mesh = meshScene.CurrentMesh;
	var saved_curvature_file  = mesh.CreateMultiScaleCurvatureMap(2, 7, 20, 40);
	// the obtained curvature file (automatically saved) is needed for CBA - store in array, one for each subject (per hemisphere)
	BrainVoyager.PrintToLog("Curvature file: " + saved_curvature_file);
}

function RunRigidCBAStep()
{
	var ok;
	var docVMR = BrainVoyager.ActiveDocument; if(docVMR == undefined) return;
	var meshScene = docVMR.MeshScene; if(meshScene == undefined) return;
	// the mesh for rigid alignment must be a sphere (SPH file)
	// to ensure that we have a sphere mesh, we can create one
	meshScene.CreateStandardSphereMesh();
	var mesh = meshScene.CurrentMesh;

	meshScene.ClearGroupCBACurvatureFiles();
	var NSubjects = 5;
	var CurvatureFile, TargetCurvatureFile;
	for(i=0; i<NSubjects; i++) // loop over subjects (per hemisphere)
	{
		// for demonstration, we simply hard-code files here - better to use/prepare array
		if(i == 0) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/AA/AA_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 1) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/MR/MR_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 2) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/PD/PD_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 3) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/RM/RM_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 4) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/SG/SG_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		
		ok = meshScene.AddCurvatureFileForGroupCBA(CurvatureFile);
		if(!ok) // provided file not found
			return;

		if(i == 0) TargetCurvatureFile = CurvatureFile; // we here use first case for rigid alignment target (any should be fine)
	}
	var SphericalTargetCurvatureFile = mesh.CreateSphericalCoordinatesMapFromSMP(TargetCurvatureFile);
	if(SphericalTargetCurvatureFile == "") // we get empty string if it did not work
		return;

	ok = meshScene.RunRigidCBA(SphericalTargetCurvatureFile);
	// the result of rigid alignment is automatically saved in a ".rga" file that will be used by subsequent CBA when using same input curvature SMP files
}

function RunFullCBA()
{
	var ok;
	var docVMR = BrainVoyager.ActiveDocument; if(docVMR == undefined) return;
	var meshScene = docVMR.MeshScene; if(meshScene == undefined) return; // creates surface window if not present

	meshScene.ClearGroupCBACurvatureFiles(); // if this is run after rigid-CBA, we would not strictly fill curvature SMP list again
	var NSubjects = 5;
	var CurvatureFile;
	for(i=0; i<NSubjects; i++) // loop over subjects (per hemisphere)
	{
		// for demonstration, we simply hard-code files here - better to use/prepare array
		if(i == 0) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/AA/AA_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 1) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/MR/MR_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 2) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/PD/PD_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 3) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/RM/RM_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		if(i == 4) CurvatureFile = BrainVoyager.PathToSampleData + "CortexBasedAlignment/SG/SG_TAL_LH_RECOSM_SPH_CURVATURE.smp";
		
		ok = meshScene.AddCurvatureFileForGroupCBA(CurvatureFile);
		if(!ok) // provided file not found
			return;
	}
	ok = meshScene.RunCBA(); if(!ok) return;
	
	// these convenience functions can only be called after CBA (with internally stored set of .SRF, .SSM, .SMP file names):
	// create group curvature map indicating quality of alignment, saves also ".cal" file that can be used in CBA dialog
	ok = meshScene.CreateAverageCurvatureGroupMap(); 
	// create folded group average cortex using created alignment (SSM) files, saves also ".sal" file that can be used in CBA dialog
	// the function only works if names of folded SPH meshes can be derived from	curvature names by replacing trailing substring with "_RECOSM_SPH.srf" 
	ok = meshScene.CreateAverageFoldedGroupMesh();
}

//CreateSphereFromFoldedMesh();
//MapSphereGetCurvature();
//RunRigidCBAStep();
RunFullCBA();

// helpers
function removeLastUnderscoreSegment(filename) {
	var lastsegmentlength = getlastsegmentlength(filename);
	var name = "";
	for (i=0; i<(filename.length-lastsegmentlength); i++) {
		letter = filename[i];
		name += letter;
	}	
	return name;
}

function getlastsegmentlength(filename) {
	var lastsegmentlength, letter, underscorepos;
	for (i=filename.length; i>0; i--) {
		letter = filename[i];
		if (letter == "_") {
			underscorepos = i;					
			break;
		}	
	}
	lastsegmentlength = filename.length - underscorepos;
	return lastsegmentlength;
}
