//
// This script shows how a mesh can be smoothed and inflated.
// It is assumed that a "*_RECO.srf" mesh is availalbe (e.g. loaded)
//
//  Created by Rainer Goebel, May 2013
//

function MorphMesh()
{
	var ok;
	var docVMR = BrainVoyager.ActiveDocument;
	if(docVMR == undefined) return;
	var mesh = docVMR.CurrentMesh;
	if(mesh == undefined) return;

	mesh.MorphingUpdateInterval = 25;
	//mesh.SmoothMesh(30, 0.07); // standard smoothing - mesh will shrink
	mesh.SmoothRecoMesh(50, 0.07); // special "high-frequency" smoothing removing jags of reconstructed voxel borders without shrinking mesh 
	mesh.CalculateCurvature();
	mesh.SmoothCurrentMap(5);
	mesh.MeshScene.UpdateSurfaceWindow();
	var new_name = removeextension(mesh.FileName) + "SM.srf";
	mesh.SaveAs(new_name);
	mesh.InflateMesh(500, 0.8, ""); // if "" used for 3rd param (area reference mesh), the current mesh at the moment the fn is called is used to calculate ref mesh area
}

MorphMesh();

// helpers
function removeextension(filename) {
	var extlength = getextensionlength(filename);
	var name = "";
	for (i=0; i<(filename.length-extlength); i++) {
		letter = filename[i];
		name += letter;
	}	
	return name;
}

function getextensionlength(filename) {
	var extensionlength, letter, dotpos;
	for (i=filename.length; i>0; i--) {
		letter = filename[i];
		if (letter == ".") {
			dotpos = i;					
			break;
		}	
	}
	extensionlength = filename.length - dotpos;
	return extensionlength;
}
