
// This simple script shows how VTC files linked ot a VMR document can be preprocessed
// This is especially helpful if VTCs are created with no (or modest, i.e. 4mm) spatial smoothing
// but when spatial smoothing (e.g. with FWHM of 8-10mm) is desired for group studies.
// In this case one could smooth the original FMR and create a second VTC file. It is, however,
// much more efficient to smooth the VTC file directly with an appropriate kernel as shown here
// While spatial smoothing is probably the most useful scenario of VTC smoothing, the
// code below shows how to call all available preprocessing options.
//
// To prepare this script, load a VMR and link a VTC - or add the appropriate script commands

var docVMR = BrainVoyager.ActiveDocument;

BrainVoyager.PrintToLog("Current VTC file: " + docVMR.FileNameOfCurrentVTC);  // show name of current VTC

// now smooth VTC with a large kernel of 10 mm:
//
docVMR.SpatialGaussianSmoothing( 10, "mm" );    // FWHM value and unit ("mm" or "vx")

BrainVoyager.PrintToLog("Name of spatially smoothed VTC file: " + docVMR.FileNameOfCurrentVTC);

// now we could do a linear trend removal (see code in comments)
// since high-pass temporal filter (see below) includes LTR, we skip this here
//docVMR.LinearTrendRemoval();    // FWHM value and unit ("mm" or "vx")
//BrainVoyager.PrintToLog("Name of VTC file without linear trends: " + docVMR.FileNameOfCurrentVTC);

// now perform temporal high-pass filter
//
docVMR.TemporalHighPassFilter(3, "cycles");    // HP alue and unit ("cycles" or "Hz")
BrainVoyager.PrintToLog("Name of VTC file without linear trends: " + docVMR.FileNameOfCurrentVTC);

// now perform Gaussian temporal smoothing
//
docVMR.TemporalGaussianSmoothing(3, "dps");    // FWHM value and unit ("d" or "dps" (data points) or "s" or "secs" (seconds))
BrainVoyager.MessageBox("Name of temporally smoothed VTC file: " + docVMR.FileNameOfCurrentVTC);

// Note that all intermediate VTC files are kept on disk. In order to remove no longer needed files, use
// the file access script routines (see "UsingCustomFiles.js" script) 